/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include <qbitmap.h>
#include <qpainter.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qlayout.h>
//---------------------------------------------------------------------------
#include <kconfigbase.h>
//---------------------------------------------------------------------------
#include "asciil.h"
#include "scheditfontprefs.h"
#include "ScheditBitmap.h"
#include "colourvaluepref.h"
//---------------------------------------------------------------------------
QStringList ScheditBitmap::slRotation;
QStringList ScheditBitmap::slSide;
QStringList ScheditBitmap::slFunction;
int ScheditBitmap::iBitmapCount = 0;
ScheditBitmap::prefsList   ScheditBitmap::prefPages;
//---------------------------------------------------------------------------
QCheckBox		* ScheditBitmap::cbShowVal = NULL;
QCheckBox		* ScheditBitmap::cbShowDes = NULL;
QCheckBox		* ScheditBitmap::cbShowPin = NULL;
QCheckBox		* ScheditBitmap::cbShowFun = NULL;
ColourValuePref * ScheditBitmap::lpBmp	   = NULL;
//---------------------------------------------------------------------------
ScheditBitmap::ScheditBitmap ( QCanvas * 		 canvas,
							   unsigned char * & puc )
 			 : CanvasPixmap  (           		 canvas ) {
	SectionData * psd;
	int         iCount;
	int         iLength;
    int         iDataLineCount;
    int         iSectionCount;
	int			iLines;

	iLength = sizeof ( bmphdr );
	memcpy ( &bmphdr, puc, iLength );
	puc += iLength;

	setX ( bmphdr.usBitmapX );
	setY ( bmphdr.usBitmapY );
	Asciil al ( bmphdr.asciilDes );
	qsDesig = al.toQString ();
	
    iSectionCount = bmphdr.ucSecCount;
    apsd = new SectionData * [iSectionCount];
    iDataLineCount = iSectionCount >> 1;
    if ( iSectionCount % 2 ) iDataLineCount++;
    asdl = new SectionDataLine [iDataLineCount];
    iLength = sizeof (SectionDataLine);
    for ( int iDataLineIndex = 0, iSectionIndex = 0;
          iDataLineIndex < iDataLineCount;
          iDataLineIndex++, puc += iLength ) {
        memcpy ( &asdl [iDataLineIndex ],
                 puc,
                 iLength );
        apsd [iSectionIndex++] = &asdl [iDataLineIndex].sdPlus0;
        if ( iSectionIndex < iSectionCount )
            apsd [iSectionIndex++] = &asdl [iDataLineIndex].sdPlus1;
    } /* endfor */
    iSection = (int) bmphdr.ucSecIndex;
//    iSection--;

// get pointer to image data
	psd = apsd [iSection - 1];

// Basic geometry
	iRowCount = psd->getBmpRows();
	iColCount = psd->getBmpCols();
	
	iPinCount = (int) bmphdr.ucPins;
// Pin Records (16 bytes each)
	iCount = (int) bmphdr.ucPins;
	pins = new PinRecord * [iCount];
	for ( int iPinIndex = 0;
		  iPinIndex < iCount;
		  iPinIndex++ ) {
		pins [iPinIndex] = new PinRecord ( puc, this );
	} /* endfor */
			
	pucImageData = (unsigned char *) puc;
	Render (psd);
	
// Update pointer to take all bitmap data
	puc = pucImageData;
	pucImageData = NULL;        // We do not own this data (destructor)
	iLength = bmphdr.usRowCount * bmphdr.usBytesPerRow;     // bytes
	iLines = iLength >> 4;                                  // lines
	if ( iLength % 0x10 ) iLines++;                         // remainder
	iLength = iLines << 4;
	puc += iLength;	
	
	show ();
	
// Static data
// Store these string lists once only
	if ( iBitmapCount++ ) return;

// Create string lists and store data
	slRotation << "None"
			<< "90 anticlockwise"
			<< "180"
			<< "90 clockwise";

	slSide << "Left"
			<< "Bottom"
			<< "Right"
			<< "Top";

	slFunction << "IN"
			<< "OUT"
			<< "I/O"
			<< "HiZ"
			<< "OC"
			<< "Pwr"
			<< "Passive";
}
//---------------------------------------------------------------------------
ScheditBitmap::~ScheditBitmap () {
// bmp is deleted in the destructor for canvasBitmap (ancestor)	
	
// Delete the section data lines
	delete [] asdl;

// Delete the section data array
	delete [] apsd;

// Delete the pin array
	for ( int iPinIndex = 0;
			 iPinIndex < (int) bmphdr.ucPins;
			 iPinIndex++ )
		delete pins [iPinIndex];

	delete [] pins;

// If required
	//if ( pucImageData ) delete [] pucImageData;
}
//---------------------------------------------------------------------------
void ScheditBitmap::TransformPoint ( QPoint& qpPinData ) {
	QWMatrix wm;

// Firstly, rotate
	if ( orthoAngle ) {
		wm = wmOrtho [orthoAngle];
		qpPinData = wm.map ( qpPinData );
	} /* endif */
	
// Secondly, flip
	if ( bFlippedX )
		qpPinData = wmFlipH.map ( qpPinData );

	if ( bFlippedY )
		qpPinData = wmFlipV.map ( qpPinData );
}
//---------------------------------------------------------------------------
// Used to determine where a device side ends up on the screen
void ScheditBitmap::TransformSide ( PinRecord::eScheditSide& eSide ) {
    int iSide;
    int iRotation;

    // moving back to recordPin
    iSide = (int) eSide;
    iRotation = orthoAngle;
    iSide += iRotation;

// Bound iSide from 0 to 3
    while ( iSide < 0 ) iSide += 4;
    iSide &= 0x03;

// Check flip
    switch ( iSide ) {
    case PinRecord::eBorderTop:
    case PinRecord::eBorderBottom:
        if ( !bFlippedY ) break;
        iSide = iSide ^ 0x02;
        break;

    case PinRecord::eBorderLeft:
    case PinRecord::eBorderRight:
        if ( !bFlippedX ) break;
        iSide = iSide ^ 0x02;
        break;

    default:
        break;
    } /* endif */

    iSide &= 0x03;
    eSide = (PinRecord::eScheditSide) iSide;
}
//---------------------------------------------------------------------------
// make Bitmap in desired orientation
//---------------------------------------------------------------------------
void ScheditBitmap::Render ( SectionData * psd ) {
    QRect         rect;
	QPoint        tp;
	QPen pen ( Qt::black, 1, Qt::SolidLine );				// database 

// Setup the orientation of the device ( flip, rotation )
	bFlippedY  = false;                        // FlipV is done by 180deg+HFlip
    bFlippedX  = bmphdr.orient.usFlip;
	setRotation ( (eRotate) bmphdr.orient.usRot );

// Rationalise FlipV (implemeted as 180 deg retation + Horizontal flip !)
    if ( bFlippedX )
    	switch (orthoAngle) {
        case anti180:
            orthoAngle = none;
            bFlippedX = false;
            bFlippedY = true;
            break;

        case none:
            break;

    	case anti90:
            break;

	    case anti270:
            break;
        } /* endswitch */

// Set pen ?
	if ( psd->isBmp() ) {
        pucSectionImageData  = pucImageData;
        pucSectionImageData += psd->getBmpDataOffs ();
		bmp = new QBitmap ( iColCount, 
                            iRowCount, 
							pucSectionImageData + 1, 
							FALSE );

    } else {	// Make a clear bitmap, draw a box in it
		QPen penDefault (  Qt::color1, 1 );
		bmp = new QBitmap ( iColCount, iRowCount, TRUE );   // Make a clear bitmap
		QPainter pixPaint(bmp);								// create the bitmap
		pixPaint.setPen ( penDefault );
		pixPaint.drawRect ( 0, 0, 
							iColCount - 1, iRowCount - 1 );
	} /* endif */
	bmp->setMask ( *bmp );			// make transparent
	setPixmap ( bmp );
}
//---------------------------------------------------------------------------
// Draw pins
void ScheditBitmap::drawTransformed ( QPainter& painter ) {
	int 		 iPin;
	PinRecord  * pPin;
	int iDelta;
	QPixmap pm = *pixmap (); 
	int iValue = lpBmp->getValue ();
	QPen	penBitmap;

	penBitmap.setColor ( lpBmp->getColour () );
	penBitmap.setWidth ( iValue );
	painter.setPen ( penBitmap );

// Draw wires, invert circles and clock triangles
    for ( iPin = 0; iPin < iPinCount; iPin++ ) {
		pPin = pins [iPin];
		if ( pPin->sectionMismatch (iSection) ) continue;
        pPin->RenderPin ( painter );
	} /* endfor */
	
	// draw bitmap again by width
	// The ancestor manages the base image
	
	// value 2 - draw down & left evens
	// value 3 - draw up & right  odds
	
	for ( int iIndex = 1; iIndex < iValue; iIndex++ ) {
		iDelta = iIndex >> 1;
		iDelta++;
		if ( iIndex & 1 ) iDelta *= -1;
		 
		painter.drawPixmap ( iDelta, 0, pm );
		painter.drawPixmap ( 0, iDelta, pm );
		painter.drawPixmap ( iDelta, iDelta, pm );
	} /* endfor */
}
//---------------------------------------------------------------------------
// We want the canvas in normal display
// we may alter it if we need to, but we promise to put it back after
void ScheditBitmap::drawUntransformed ( QPainter& painter ) {
	int 		 iPin;
	PinRecord  * pPin;
	bool		 bVertText = false;
	QPoint 		 qpPin;
	PinRecord::eScheditSide side;
	ScheditPrefs * lp;
			
// Draw Designation and Value
	if ( cbShowDes->isChecked () ) {
		lp = (ScheditFontPrefs *) prefPages.at (1);	// "Designator"
		lp->setupPainter ( painter );
		painter.drawText ( bmphdr.usDesX - bmphdr.usBitmapX, 
						   bmphdr.usDesY - bmphdr.usBitmapY, 
		 				   qsDesig );
	} /* endif */

	if ( cbShowVal->isChecked () ) {
		Asciil  al ( bmphdr.asciilVal );
		QString qs = al.toQString ();

		lp = (ScheditFontPrefs *) prefPages.at (0);	// "Value"
		lp->setupPainter ( painter );
		painter.drawText ( bmphdr.usValX - bmphdr.usBitmapX, 
						   bmphdr.usValY - bmphdr.usBitmapY, 
		 				   qs );
	} /* endif */

// Draw pin numbers (never rotated)
	if ( cbShowPin->isChecked () ) {	
		lp = (ScheditFontPrefs *) prefPages.at (2);	// "Number"
		lp->setupPainter ( painter );
		for ( iPin = 0; iPin < iPinCount; iPin++ ) {
			pPin = pins [iPin];
			if ( pPin->sectionMismatch (iSection) ) continue;
			qpPin = pPin->getPoint ();
			TransformPoint ( qpPin );
			side = pPin->getSide ();
			TransformSide ( side );
			pPin->PinNum ( painter, 
							qpPin,	
							side );	
		} /* endfor */
	} /* endif */
	
// draw text ( horizontal )
	if ( cbShowFun->isChecked () ) {
		lp = (ScheditFontPrefs *) prefPages.at (3);	// "Function"
		lp->setupPainter ( painter );
		for ( iPin = 0; iPin < iPinCount; iPin++ ) {
			pPin = pins [iPin];
			if ( pPin->sectionMismatch (iSection) ) continue;
			side = pPin->getSide ();
			TransformSide ( side );
			switch ( side ) {
			case PinRecord::eBorderLeft:
			case PinRecord::eBorderRight:
				qpPin = pPin->getPoint ();
				TransformPoint ( qpPin );
				pPin->PinStr ( painter, qpPin, side );
				break;
	
			case PinRecord::eBorderTop:
			case PinRecord::eBorderBottom:
				bVertText = true;
				break;
			} /* endswitch */
		} /* endfor */
	
		if ( bVertText ) {		// no rotation for text required

// draw text ( vertical ) : required by top & bottom connections
			painter.save ();
			painter.rotate ( -90 );
			for ( iPin = 0; iPin < iPinCount; iPin++ ) {
				pPin = pins [iPin];
				if ( pPin->sectionMismatch (iSection) ) continue;
				side = pPin->getSide ();
				TransformSide ( side );
				switch ( side ) {
				case PinRecord::eBorderLeft:
				case PinRecord::eBorderRight:
					break;
		
				case PinRecord::eBorderTop:
				case PinRecord::eBorderBottom:
					qpPin = pPin->getPoint ();
					TransformPoint ( qpPin );
					pPin->PinStr ( painter, qpPin, side );
					break;
				} /* endswitch */
			} /* endfor */
			painter.restore ();
		} /* endif */
	} /* endif */
}
//---------------------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int ScheditBitmap::rtti () const {
	return 1011;
}
//---------------------------------------------------------------------------
int ScheditBitmap::getExtension ( void ) {
    int iExtension;

	switch ( bmphdr.ucDisplayType ) {
    case eScheditNormal:
    case eScheditConnector:
        iExtension = 31;
        break;

    case eScheditDevice:
    case eScheditPower:
    case eScheditSymbol:
        iExtension = 11;
        break;
    } /* endswitch */

    return iExtension;
}
//---------------------------------------------------------------------------
// Other stuff
//---------------------------------------------------------------------------
QSize ScheditBitmap::getBmpSize ( void ) {
	return QSize ( iColCount, iRowCount );
}
//---------------------------------------------------------------------------
// SectionData
//---------------------------------------------------------------------------
int ScheditBitmap::SectionData::getBmpDataOffs ( void ) {
    unsigned int uiAddr;

    uiAddr = (unsigned int) (ucA -1);
    uiAddr <<= 4;
    return (int) uiAddr;
}
//---------------------------------------------------------------------------
int ScheditBitmap::SectionData::getBmpRows ( void ) {
    int iRows;

    iRows = ucRowBlocks * 10;
    iRows++;
    return iRows;
}
//---------------------------------------------------------------------------
int ScheditBitmap::SectionData::getBmpCols ( void ) {
    int iCols;

    iCols = ucColBlocks * 10;
    iCols++;
    return iCols;
}
//---------------------------------------------------------------------------
bool ScheditBitmap::SectionData::isBmp ( void ) {
    return ( ucA != 0 );
}
//---------------------------------------------------------------------------
// Preference Pages
//---------------------------------------------------------------------------
void ScheditBitmap::readProperties ( KConfig * config) {	
	ScheditPrefs * lp;
	
	for ( lp = prefPages.first (); lp; lp = prefPages.next () )
		lp->readGroup ( config ); 
}
//---------------------------------------------------------------------------
void ScheditBitmap::writeProperties ( KConfig * config ) {	
	ScheditPrefs * lp;
	
	for ( lp = prefPages.first (); lp; lp = prefPages.next () )
		lp->writeGroup ( config ); 
}
//---------------------------------------------------------------------------
// Loads various different ScheditPage descendants
//---------------------------------------------------------------------------
void ScheditBitmap::setup ( KDialogBase * dlg ) {	
	QString 		qsRoot = "Bitmap";
	QString 		qsTitle;
	QString 		qsKey;
	QString 		qsKeyName;
	ScheditPrefs *  lp;
	unsigned int 	uiIndex;
	unsigned int    uiCount;
	QStringList 	slPages;
		
// Font settings 
	slPages << "Value"
			<< "ID"
			<< "Pin"
			<< "Function";
	uiCount = slPages.count ();
	
	for ( uiIndex = 0; uiIndex < uiCount; uiIndex++ ) {
		qsKey = slPages [uiIndex].ascii ();
		qsTitle.sprintf ( "%s %s", 
						  qsRoot.ascii (), 
						  qsKey.ascii () );
		qsKeyName.sprintf ( "%s.%s", 
							qsRoot.ascii (), 
							qsKey.ascii () );
		lp = (ScheditPrefs *) new ScheditFontPrefs ( dlg,
													 qsTitle,
													 qsKeyName );
		prefPages.append ( lp );
	} /* endfor */
	
// Add the bitmap display page
	slPages << "Colour";
	qsKey = slPages [uiIndex].ascii ();
	qsTitle.sprintf ( "%s %s", 
					  qsRoot.ascii (), 
					qsKey.ascii () );
	qsKey.sprintf ( "%s.%s", 
					qsRoot.ascii (), 
				  "Diagram" );
	lpBmp = new ColourValuePref ( dlg, 
									qsTitle, 
									qsKey);
	prefPages.append ( (ScheditPrefs *) lpBmp );

	lpBmp->adjustForCheckBoxes ();	
	
// Create layout	
	QVBoxLayout * vbLayout = lpBmp->addLayout ();
	
// Create checkboxes
	cbShowVal = lpBmp->makeCb ( "Show &Value" );
	lpBmp->layoutCb ( cbShowVal, vbLayout );
	cbShowDes = lpBmp->makeCb ( "Show &ID" );
	lpBmp->layoutCb ( cbShowDes, vbLayout );
	cbShowPin = lpBmp->makeCb ( "Show &Pin Numbers" );
	lpBmp->layoutCb ( cbShowPin, vbLayout );
	cbShowFun = lpBmp->makeCb ( "Show &Function Names" );
	lpBmp->layoutCb ( cbShowFun, vbLayout );

/*	// For some reason, we could not do this in "ColourValuePref"
	cbShowVal = lpBmp->createCb ( "Show Value", 		 vbLayout );
	cbShowDes = lpBmp->createCb ( "Show ID", 			 vbLayout );
	cbShowPin = lpBmp->createCb ( "Show Pin Numbers",    vbLayout );
	cbShowFun = lpBmp->createCb ( "Show Function Names", vbLayout );
*/
		
// Create display
	QCanvas * canvas = lpBmp->getCanvas ();
	unsigned char * puc;
	
	unsigned char dev_BC549 [384] = {
		0x70,0x02,0x51,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x10,0x00,
		0x41,0x05,0x42,0x43,0x35,0x34,0x39,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x29,0x00,
		0x42,0x00,0x00,0x3d,0x00,0x1a,0x00,0x3d,0x00,0x36,0x00,0x01,0x01,0x04,0x04,0x01,
		0x43,0x03,0x02,0x02,0x00,0x05,0x54,0x30,0x2d,0x39,0x32,0x00,0x00,0x00,0x00,0x00,
		0x01,0x00,0x04,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x01,0x60,0x01,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,
		0x01,0x60,0x01,0x43,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x03,
		0x01,0x60,0x01,0x45,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x03,
		0x00,0x00,0x00,0xff,0x82,0x00,0x00,0x00,0x07,0x00,0x72,0x00,0x00,0x00,0x18,0x00,
		0x0e,0x00,0x00,0x00,0x60,0x00,0x07,0x00,0x00,0x00,0x80,0x00,0x04,0x80,0x00,0x01,
		0x00,0x00,0x08,0x40,0x00,0x02,0x00,0x00,0x10,0x20,0x00,0x04,0x07,0x00,0x20,0x10,
		0x00,0x08,0x07,0x00,0x40,0x08,0x00,0x10,0x07,0x00,0x80,0x04,0x00,0x10,0x07,0x01,
		0x00,0x04,0x00,0x20,0x07,0x02,0x00,0x02,0x00,0x20,0x07,0x04,0x00,0x02,0x00,0x40,
		0x07,0x08,0x00,0x01,0x00,0x40,0x07,0x10,0x00,0x01,0x00,0x40,0x07,0x20,0x00,0x01,
		0x00,0x80,0x07,0x40,0x00,0x00,0x80,0x80,0x07,0x80,0x00,0x00,0x80,0x80,0x07,0x00,
		0x00,0x00,0x80,0x80,0x07,0x00,0x00,0x00,0x80,0xff,0xff,0x00,0x00,0x00,0x80,0x80,
		0x07,0x00,0x00,0x00,0x80,0x80,0x07,0x00,0x00,0x00,0x80,0x80,0x07,0x00,0x00,0x00,
		0x80,0x80,0x07,0x00,0x00,0x00,0x80,0x40,0x07,0x80,0x00,0x01,0x00,0x40,0x07,0x40,
		0x00,0x01,0x00,0x40,0x07,0x20,0x00,0x01,0x00,0x20,0x07,0x10,0x00,0x02,0x00,0x20,
		0x07,0x08,0x00,0x02,0x00,0x10,0x07,0x04,0x00,0x04,0x00,0x10,0x07,0x02,0x20,0x04,
		0x00,0x08,0x07,0x01,0x60,0x08,0x00,0x04,0x07,0x00,0xf0,0x10,0x00,0x02,0x00,0x01,
		0xf0,0x20,0x00,0x01,0x00,0x03,0xf8,0x40,0x00,0x00,0x80,0x00,0xf8,0x80,0x00,0x00,
		0x60,0x00,0x3d,0x00,0x00,0x00,0x18,0x00,0x0e,0x00,0x00,0x00,0x07,0x00,0x72,0x00,
		0x00,0x00,0x00,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	};
	puc = dev_BC549;
	new ScheditBitmap ( canvas, puc );
	
	unsigned char dev_4029 [336] = {
		0x70,0x04,0x55,0x33,0x30,0x32,0x8b,0x46,0x0c,0x2d,0x4e,0x00,0xa0,0x00,0x06,0x00,
		0x41,0x04,0x34,0x30,0x32,0x39,0x36,0x74,0x45,0x2d,0x95,0x00,0x00,0x00,0x00,0x00,
		0x42,0x00,0x00,0x00,0x01,0x1a,0x00,0x00,0x01,0x36,0x00,0x01,0x01,0x06,0x0b,0x00,
		0x43,0x10,0x02,0x02,0x48,0x05,0x44,0x49,0x50,0x31,0x36,0xff,0x76,0x0e,0xff,0x76,
		0x00,0x00,0x06,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x01,0x00,0x01,0x34,0x00,0x00,0x00,0x02,0x50,0x30,0x00,0x00,0x00,0x00,0x00,0x01,
		0x01,0x10,0x01,0x36,0x00,0x00,0x00,0x02,0x51,0x30,0x00,0x00,0x00,0x00,0x02,0x01,
		0x01,0x00,0x02,0x31,0x32,0x00,0x00,0x02,0x50,0x31,0x00,0x00,0x00,0x00,0x00,0x02,
		0x01,0x10,0x02,0x31,0x31,0x00,0x00,0x02,0x51,0x31,0x00,0x00,0x00,0x00,0x02,0x02,
		0x01,0x00,0x02,0x31,0x33,0x00,0x00,0x02,0x50,0x32,0x00,0x00,0x00,0x00,0x00,0x03,
		0x01,0x10,0x02,0x31,0x34,0x00,0x00,0x02,0x51,0x32,0x00,0x00,0x00,0x00,0x02,0x03,
		0x01,0x00,0x01,0x33,0x00,0x00,0x00,0x02,0x50,0x33,0x00,0x00,0x00,0x00,0x00,0x04,
		0x01,0x10,0x01,0x32,0x00,0x00,0x00,0x02,0x51,0x33,0x00,0x00,0x00,0x00,0x02,0x04,
		0x01,0x01,0x01,0x35,0x00,0x00,0x00,0x02,0xc3,0xc9,0x00,0x00,0x00,0x00,0x00,0x06,
		0x01,0x11,0x01,0x37,0x00,0x00,0x00,0x02,0xc3,0xcf,0x00,0x00,0x00,0x00,0x02,0x06,
		0x01,0x02,0x02,0x31,0x35,0x00,0x00,0x03,0x43,0x4c,0x4b,0x00,0x00,0x00,0x00,0x07,
		0x01,0x00,0x01,0x31,0x00,0x00,0x00,0x02,0x50,0x45,0x00,0x00,0x00,0x00,0x00,0x08,
		0x01,0x00,0x01,0x39,0x00,0x00,0x00,0x03,0x42,0x2f,0xc4,0x00,0x00,0x00,0x00,0x09,
		0x01,0x00,0x02,0x31,0x30,0x00,0x00,0x03,0x55,0x2f,0xc4,0x00,0x00,0x00,0x00,0x0a,
		0x01,0x54,0x02,0x31,0x36,0x00,0x00,0x03,0x56,0x43,0x43,0x00,0x00,0x00,0x03,0x00,
		0x01,0x54,0x01,0x38,0x00,0x00,0x00,0x03,0x47,0x4e,0x44,0x00,0x00,0x00,0x01,0x00
	};
	puc = dev_4029;
	new ScheditBitmap ( canvas, puc );
};
//---------------------------------------------------------------------------
QFontMetrics& ScheditBitmap::getNumFontMetrics ( void ) {
	ScheditFontPrefs * lp = (ScheditFontPrefs *) prefPages.at ( 2 );	
	
	return lp->getFontMetrics ();
}
//---------------------------------------------------------------------------
QFontMetrics& ScheditBitmap::getFunFontMetrics ( void ) {
	ScheditFontPrefs * lp = (ScheditFontPrefs *) prefPages.at ( 3 );	
	
	return lp->getFontMetrics ();
}
//---------------------------------------------------------------------------
QString ScheditBitmap::getID (void) {
	return qsDesig;	
}
//---------------------------------------------------------------------------
