/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#ifndef ScheditBitmap_H
#define ScheditBitmap_H
//---------------------------------------------------------------------------
#include <canvaspixmap.h>
#include <qcheckbox.h>
//---------------------------------------------------------------------------
#include "pinrecord.h"
#include "scheditjunction.h"
#include "scheditfontprefs.h"
//---------------------------------------------------------------------------
class QBitmap;
class ScheditPrefs;
class KDialogBase;
class KConfig;
class ColourValuePref;
class QCheckBox;
//---------------------------------------------------------------------------
/**
	@author James Fuller,,, <james@slakware12>
*/
class ScheditBitmap : public CanvasPixmap {
private:
	QString qsDesig;
	
	typedef QPtrList<ScheditPrefs> prefsList;
	static prefsList prefPages;
	static ColourValuePref * lpBmp;
	static QCheckBox * cbShowVal;
	static QCheckBox * cbShowDes;
	static QCheckBox * cbShowPin;
	static QCheckBox * cbShowFun;

public:
    ScheditBitmap ( QCanvas * canvas,
				    unsigned char * & puc );

    ~ScheditBitmap();

	virtual int rtti () const;

	void drawTransformed ( QPainter& painter );

	void drawUntransformed ( QPainter& painter );

	int getExtension ( void );

	QSize getBmpSize ( void );
	
	QString getID (void);
	
	static void setup ( KDialogBase * dlg );
	static void readProperties  ( KConfig * config);	
	static void writeProperties ( KConfig * config );	

	static QFontMetrics& getNumFontMetrics ( void );
	static QFontMetrics& getFunFontMetrics ( void );
	
private:
    typedef enum {
        eScheditNormal,
        eScheditDevice,
        eScheditPower,
        eScheditSymbol,
        eScheditConnector,
    } eScheditDisplayType;

#pragma pack (push,1)
	typedef struct {
		unsigned short  usRot:2;
		unsigned short  usFlip:1;
		unsigned short  usQue1:11;
	} Orient;

//    typedef struct {
//        unsigned char bHidePinNames:1;
//        unsigned char bHideValueString:1;
//        unsigned char ucQue2:6;
//    } ViewStyle;

	typedef struct {
		unsigned char  ucType;
		char           asciilDes [11];
		unsigned short usBitmapX;
		unsigned short usBitmapY;
		char 		   cA;					// 'A'
		char           asciilVal [11];
		unsigned short usBytesPerRow;
		unsigned short usRowCount;
		char 		   cB;					// 'B'
		Orient         orient;
		unsigned short usDesX;
		unsigned short usDesY;
		unsigned short usValX;
		unsigned short usValY;

        unsigned char ucSecIndex;
        unsigned char ucSecCount;
        unsigned char ucColBlocks;
        unsigned char ucRowBlocks;
        unsigned char ucDisplayType;

		char 		   cC;					// 'C'
		unsigned char  ucPins;
		unsigned char  ucQue3[3];		    // 02 02 00
		char		   asciilPkg [11];
	} BitmapHdr;

    typedef struct {
        unsigned char ucA;
        unsigned char ucB;
        unsigned char ucColBlocks;
        unsigned char ucRowBlocks;
        int  getBmpDataOffs ( void );
        int  getBmpRows     ( void );
        int  getBmpCols     ( void );
        bool isBmp          ( void );
    } SectionData;

    typedef struct {
        SectionData sdPlus0;
        SectionData sdPlus1;
        unsigned char acuQueI [8];
    } SectionDataLine;
#pragma pack (pop)

	QBitmap * bmp;

	BitmapHdr           bmphdr;	    // Four lines of 16 bytes
	SectionDataLine   * asdl;           // one to n lines each holding two sections
    SectionData      ** apsd;
	PinRecord        ** pins; 	        // Zero to n lines of data
    unsigned char     * pucImageData;
    unsigned char     * pucSectionImageData;
    unsigned char       ucSecCount;
    int                 iSection;
	int 		 		iColCount;
	int 		 		iRowCount;
    int          		iPinCount;

//	static QPen  penBitmap;

	static int iBitmapCount;
	static QStringList slFunction;
	static QStringList slSide;
	static QStringList slRotation;
	
	void TransformPoint ( QPoint& qpPinData );
	void TransformSide ( PinRecord::eScheditSide& eSide );
	void Render ( SectionData * psd );
};
//---------------------------------------------------------------------------

#endif
