//---------------------------------------------------------------------------
#include <qstring.h>
#include "asciil.h"
//---------------------------------------------------------------------------
/**
 * Default Creator
 * @param  none
 */
Asciil::Asciil ( void ) {
	char * psz = (char *) "<unassigned>";
	int iLength = strlen ( psz );

	pcData = new char [iLength + 1];
	pcData [0] = (char) iLength;
	memcpy ( pcData + 1, psz, iLength );
}
//---------------------------------------------------------------------------
/**
 * Creator from ASCIIZ string
 * @param pcBuf Source
 */
Asciil::Asciil ( char * pcBuf ) {
	int iLength = (int) *pcBuf;

	pcData = new char [iLength + 1];
	pcData [0] = (char) iLength;
	memcpy ( pcData + 1, pcBuf + 1, iLength );
}
//---------------------------------------------------------------------------
/**
 * Destructor
 * @param  none
 * @return None
 */
Asciil::~Asciil (void) {
	delete [] pcData;
}
//---------------------------------------------------------------------------
/**
 * Copy Creator
 * @param al Source
 */
Asciil::Asciil ( Asciil& al ) {
	int iLength = al.size ();

	pcData = new char [iLength];
	memcpy ( pcData, al.pcData, iLength );
}
//---------------------------------------------------------------------------
/**
 * Creator from QString
 * @param as Source
 */
Asciil::Asciil ( QString& as ) {
    pcData = NULL;
    fromQString (as);
}
//---------------------------------------------------------------------------
/**
 * Converter from QString
 * @param as Source
 */
void Asciil::fromQString ( QString& as ) {
	int iLength = as.length ();
	if ( iLength > 255) iLength = 255;

    if (pcData) delete [] pcData;
	pcData = new char [iLength + 1];
	pcData [0] = (char) iLength;
	memcpy ( pcData + 1, as.ascii (), iLength );
}
//---------------------------------------------------------------------------
/**
 * Converter from QString with length limit
 * @param as Source 
 * @param iLimit Length to convert
 */
void Asciil::fromQString ( QString& as, int iLimit ) {
	int iLength = as.length ();
	if ( iLength > iLimit ) iLength = iLimit;

    if (pcData) delete [] pcData;
	pcData = new char [iLength + 1];
	pcData [0] = (char) iLength;
	memcpy ( pcData + 1, as.ascii (), iLength );
}
//---------------------------------------------------------------------------
/**
 * Ptr to internal data allocation
 * @param  none
 * @return Pointer
 */
char * Asciil::memory ( void ) {
	return pcData;
}
//---------------------------------------------------------------------------
/**
 * Return size of internal memory allocation
 * @param  none
 * @return Byte count
 */
int Asciil::size ( void ) {
	return length () + 1;
}
//---------------------------------------------------------------------------
/**
 * Returns length of string
 * @param  none
 * @return int
 */
int Asciil::length ( void ) {
	int iLength = ( (unsigned int) pcData [0] ) & 0xff;
	return iLength;
}
//---------------------------------------------------------------------------
/**
 * Converter to QString
 * @param  none
 * @return QString 
 */
QString Asciil::toQString ( void ) {
	unsigned int uiLength;
	QString 	asRc;
	char  		acBuf [256];    // dynamically allocating data resulted
								// in new buffer OVER old buffer
								// CORRUPTION ! Static alloc solved issue.
	uiLength = (unsigned int) pcData [0];
    uiLength &= 0xff;
	if ( uiLength ) {
		memcpy ( acBuf, pcData + 1, uiLength);
		acBuf [uiLength] = 0;
		asRc = acBuf;

	} else
		asRc = QString ( "" );

	return asRc;
}
//---------------------------------------------------------------------------
/**
 * Insert string at index
 * @param as String to be inserted
 * @param iStart Position to insert it
 */
void Asciil::insert ( QString as, int iStart ) {
    QString asBuf = toQString ();
    asBuf.insert ( iStart, as );
    fromQString ( asBuf );
}
//---------------------------------------------------------------------------
/**
 * Delete characters from index
 * @param iStart First charater to be deleted
 * @param iLength Number of characters to be deleted
 */
void Asciil::remove ( int iStart, int iLength ) {
    QString asBuf = toQString ();
    asBuf.remove ( iStart, iLength );
    fromQString ( asBuf );
}
//---------------------------------------------------------------------------
/**
 * Replace substring
 * @param asFind String to remove
 * @param asRep Replacement string
 */
void Asciil::replace ( QString& asFind, QString& asRep ) {
    QString asBuf = toQString ();
    int iPos;
    int iLen;

    do {
        iPos = asBuf.find ( asFind );
        if ( iPos < 0 ) break;
        iLen = asFind.length ();
        remove ( iPos, iLen );
        insert ( asRep, iPos );
    } while ( true );
    fromQString ( asBuf );
}
//---------------------------------------------------------------------------
/**
 * Copy
 * @param al Source
 * @return Reference to object
 */
Asciil& Asciil::operator=(Asciil& al ) {
    int iLength = al.length();
    iLength++;
    delete [] pcData;
    pcData = new char [iLength];
    memcpy ( pcData, pcData, iLength );
    return *this;
}
//---------------------------------------------------------------------------
/**
 * Converter from ASCIIZ string
 * @param pcBuf Source
 * @return Reference to object
 */
Asciil& Asciil::operator=(char * pcBuf ) {
    unsigned int uiLength;

    uiLength = (unsigned int) pcBuf [0];
    uiLength &= 0xff;
    uiLength++;
    delete [] pcData;
    pcData = new char [uiLength];
    memcpy ( pcData, pcBuf, uiLength );

    return *this;
}
//---------------------------------------------------------------------------
/**
 * Test for Empty string
 * @param none 
 * @return bool True if empty
 */
bool Asciil::isEmpty ( void ) {
    return ( *pcData == 0 );
}
//---------------------------------------------------------------------------
/**
 * Make an Asciil string in a specified buffer  
 * @param array Buffer
 * @param iLength Size of buffer
 * @param as String to convert
 */
void Asciil::renderAsAsciil ( char *  array,
                              int     iLength,
                              QString as ) {
    int iStrLen = as.length ();

    memset ( array, 0, iLength );
    iLength--;
    if ( iStrLen > iLength ) iStrLen = iLength;
    array [0] = (char) iStrLen;
    memcpy ( array + 1, as.ascii (), iStrLen );
}
//---------------------------------------------------------------------------

