/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#include "canvasorthotext.h"
#include <qpainter.h>
//-------------------------------------------------------------------
/**
 * Creator
 * @param canvas Where the object is represented
 */
CanvasOrthoText::CanvasOrthoText ( QCanvas * canvas ) 
			   : QCanvasText ( canvas ) { 
	bVertical = false;
	bShadow   = false;
}
//-------------------------------------------------------------------
/**
 * Creator with string
 * @param t String to display
 * @param canvas Where the object is represented
 */
CanvasOrthoText::CanvasOrthoText ( QString & t, QCanvas * canvas )
			   : QCanvasText ( t, canvas ) { 
	bVertical = false;
	bShadow   = false;
}
//-------------------------------------------------------------------
/**
 * Constructor with string and font
 * @param t 			String
 * @param font 			Font
 * @param canvas 		Where the object is represented
 */
CanvasOrthoText::CanvasOrthoText ( QString & t, QFont font, QCanvas * canvas )
			   : QCanvasText ( t, font, canvas ) { 
	bVertical = false;
	bShadow   = false;
}
//-------------------------------------------------------------------
/**
 * Destructor
 */
CanvasOrthoText::~CanvasOrthoText()
{
}
//-------------------------------------------------------------------
/**
 * Copies specified parameters from object to canvas
 * @param painter 
 */
void CanvasOrthoText::setupPainter ( QPainter & painter ) {
	painter.setPen  ( color () );
	painter.setFont ( font  () );
}
//-------------------------------------------------------------------
/**
 * Draws the string onto the canvas
 * @param painter 
 */
void CanvasOrthoText::draw ( QPainter & painter ) {
	QPoint qp ( x (), y () );

	painter.save ();	
	painter.translate ( x (), y () );
	if ( bVertical ) painter.rotate ( -90 );
	setupPainter ( painter );

	if ( bShadow ) {
		QPoint shadow ( 0, 0 );
		QPen oldPen = painter.pen ();

		if ( bVertical ) 
			shadow += QPoint ( -1, 1 );

		else
			shadow += QPoint ( 1, 1 );

		painter.setPen ( Qt::black );
		painter.drawText ( shadow.x (), shadow.y (), text () );
		painter.setPen ( oldPen );
	} /* endif */

	painter.drawText ( 0, 0, text () );
	painter.restore ();
}
//-------------------------------------------------------------------
/**
 * Determine the bounding rect of the strnig at that orientation
 * @return QRect
 */
QRect CanvasOrthoText::boundingRect () const {
	QFontMetrics fm ( font () );
	QPoint qp ( x (), y () );

	QRect rect1 ( fm.boundingRect ( text () ) );

	if ( bVertical ) {
		QPoint qpWH ( rect1.height () - 1, rect1.width () - 1 );
		QPoint qpTopLeft = qp - qpWH;
		QRect rect2 ( qpTopLeft, qp );
	
		return rect2;
	}
	rect1.moveBy ( qp.x (), qp.y () );
	return rect1;
}
//-------------------------------------------------------------------
/**
 * Determine the width of the displayed string at the orientation
 * @return 
 */
int CanvasOrthoText::width () {
	return boundingRect ().width ();
}
//-------------------------------------------------------------------
/**
 * Determine the height of the string at the orientation
 * @return int
 */
int CanvasOrthoText::height () {
	return boundingRect ().height ();
}
//-------------------------------------------------------------------
/**
 * Specifies if the string to be displayed vertically
 * @param bFlag True for vertical
 */
void CanvasOrthoText::setVertical ( bool bFlag ) {
	bVertical = bFlag;
}
//-------------------------------------------------------------------
/**
 * Determines if the string is to be displayed vertically
 * @param  none
 * @return bool
 */
bool CanvasOrthoText::vertical ( void ) {
	return bVertical;
}
//-------------------------------------------------------------------
/**
 * Determines if the string is to show a shadow
 * @param  
 * @return 
 */
bool CanvasOrthoText::shadow ( void ) {
	return bShadow;
}
//-------------------------------------------------------------------
/**
 * Specifies if the string is to display a shadow
 * @param bFlag True for vertical
 */
void CanvasOrthoText::setShadow ( bool bFlag ) {
	bShadow = bFlag;
	update ();
}
//-------------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int CanvasOrthoText::rtti ( void ) const {
	return 1001;
}
//-------------------------------------------------------------------


