/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#ifndef CANVASPIXMAP_H
#define CANVASPIXMAP_H
//-------------------------------------------------------------------
#include <qcanvas.h>
#include <qwmatrix.h>
//-------------------------------------------------------------------
class QBitmap;
//-------------------------------------------------------------------
/**
*	A ScheditItem object to display a transparent monochrome bitmap
*   which may be rotated in 90 deg increments or flipped horizontally and vertically
	@author James Fuller,,, <james@slakware12>
*/
class CanvasPixmap : public QCanvasItem {
public:
	typedef enum { 
		none,
		anti90,
		anti180,
		anti270 
	} eRotate;

    CanvasPixmap(QCanvas * canvas);

    ~CanvasPixmap();

	virtual bool collidesWith ( const QCanvasItem * other ) const;

	virtual void draw ( QPainter & painter );

	virtual QRect boundingRect () const;

    virtual bool collidesWith( const QCanvasSprite*,
			       const QCanvasPolygonalItem*,
			       const QCanvasRectangle*,
			       const QCanvasEllipse*,
			       const QCanvasText* ) const;

	virtual int rtti () const;

	void flipH ( void );

	void flipV ( void );

	int width ( void );

	int height ( void );

	void rotate ( void );

	void setRotation ( eRotate orthoAngle );

	eRotate rotation ( void  );

	QPixmap * pixmap ( void );

	void setPixmap ( QPixmap * _pm );

protected:
	eRotate   orthoAngle;
	bool      bFlippedX;
	bool      bFlippedY;

	static QWMatrix wmFlipH;
	static QWMatrix wmFlipV;
	static QWMatrix wmOrtho [4];

// Extend drawing here (by overriding)
	virtual void drawTransformed ( QPainter & painter );
	virtual void drawUntransformed ( QPainter & painter );

private:
	QPixmap * pm;

	int iPixmapWidth;
	int iPixmapHeight;
};
//-------------------------------------------------------------------
#endif
