/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include <kdialogbase.h>
#include <kconfig.h>
//---------------------------------------------------------------------------
#include <qcanvas.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qcheckbox.h>
//#include <qsizepolicy.h>
//---------------------------------------------------------------------------
#include "colourvaluepref.h"
//---------------------------------------------------------------------------
extern QColor clScheditBg;
//---------------------------------------------------------------------------
// Prefs class for junction
// Colour, value, image
//---------------------------------------------------------------------------
ColourValuePref::ColourValuePref ( 	KDialogBase * dlg,
									QString 		qsTitle,
									QString 		qsKey)
				: ScheditPrefs 	( dlg, 			qsTitle, qsKey)
				, canvas 		( this )
				, pixLl 		( "Diagram", 	this )
				, view 			( this, 		"ColourValuePrefCanvas" )
				, laColour  	( "Colour", 	this )
				, dcb 			( this, 		"Colour", &laColour )
				, laValue  		( "Thickness", 	this )
				, svValue  		( 1, 5, 		&laValue, 1, this, "Thickness" ) {
	bSpinBox = true;
					
// Bold section titles
	embolden ( &laColour );
	embolden ( &laValue  );
	embolden ( &pixLl    );

	canvas.resize ( 500, 500 );
	
	connect ( &svValue, SIGNAL (valueChanged(int) ),
			this, SLOT ( render () ) );

	connect ( &dcb, SIGNAL (colorSelected(int)), 
			this, SLOT ( render () ) );

	adjustSize ();
	dcb.setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );

	svValue.setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
	
// How do we add some spare spave at the bottom ?
	QSpacerItem * si = new QSpacerItem ( 50, 50, QSizePolicy::Fixed, QSizePolicy::Maximum );
//layout.add ( si );
	QFrame * f = new QFrame ( this );
	f->setFrameRect ( QRect ( 0, 0, 100, 100 ) );
	layout.addWidget ( f );

	view.setHScrollBarMode ( QScrollView::AlwaysOff );
	view.setVScrollBarMode ( QScrollView::AlwaysOff );
	view.setCanvas ( &canvas );

	hbLayout = new QHBoxLayout ( &layout );
	hbLayout->setResizeMode ( QLayout::FreeResize ); 
	layout.remove ( &view );				// Take it out from here
	hbLayout->addWidget ( &view, 10 );		// Put it in here
	
	layout.insertSpacing ( 4, 10 );
	layout.insertSpacing ( 2, 10 );
	
// How about we let the layout itself get bigger ?
	layout.setResizeMode ( QLayout::FreeResize );
}
//---------------------------------------------------------------------------
void ColourValuePref::setSpinBoxTitle ( QString qs ) {
	laValue.setText ( qs );	
	svValue.setName ( qs );
}
//---------------------------------------------------------------------------
void ColourValuePref::showSpinBox ( bool bEnable ) {
	bSpinBox = bEnable;
	if ( bSpinBox ) {
		svValue.show ();
		laValue.show ();
	} else {
		svValue.hide ();
		laValue.hide ();
	} /* endif */		
}
//---------------------------------------------------------------------------
QColor ColourValuePref::getColour () {
	return dcb.getColour ();
}
//---------------------------------------------------------------------------
int ColourValuePref::getValue () {
	return svValue.value ();
}
//---------------------------------------------------------------------------
void ColourValuePref::renderPage () {
	canvas.resize ( view.width (), view.height () );
	view.erase ();
	canvas.setBackgroundColor ( clScheditBg );
	view.update ();	
	view.repaintContents ();	
}
//---------------------------------------------------------------------------
void ColourValuePref::readPreferences ( KConfigGroup * config ) {
	dcb.read ( config, sLastKey );
	if ( bSpinBox ) svValue.read ( config, sLastKey );
	renderPage ();
}
//---------------------------------------------------------------------------
void ColourValuePref::writePreferences ( KConfigGroup * config ) {
	dcb.write ( config, sLastKey );
	if ( bSpinBox ) svValue.write ( config, sLastKey );
}
//---------------------------------------------------------------------------
void ColourValuePref::setupPainter (QPainter& painter) {
	QColor  qc = dcb.getColour ();
	QPen 	pen   ( qc, svValue.value () );
	QBrush  brush ( qc, Qt::NoBrush );

	painter.setPen   ( pen   );
	painter.setBrush ( brush );
}
//---------------------------------------------------------------------------
QCanvas * ColourValuePref::getCanvas () {
	return &canvas;
}
//---------------------------------------------------------------------------
QVBoxLayout * ColourValuePref::addLayout ( void ) {
	QVBoxLayout * vb = new QVBoxLayout (hbLayout);
	layout.setAutoAdd ( false );
	hbLayout->setSpacing ( 10 );
	return vb;
}
//---------------------------------------------------------------------------
void ColourValuePref::layoutCb ( QCheckBox * cb, QVBoxLayout * vbLayout ) {	
	vbLayout->addWidget ( cb );		// add to new layout (provided )
}
//---------------------------------------------------------------------------
QCheckBox * ColourValuePref::createCb ( QString qs, QVBoxLayout * vbLayout ) {	
	QCheckBox * cb = makeCb ( qs );
	vbLayout->addWidget ( cb );		// add to new layout (provided )
}
//---------------------------------------------------------------------------
void ColourValuePref::adjustForCheckBoxes ( void ) {	
	view.setMaximumHeight ( 150 );
	adjustSize ();
}
//---------------------------------------------------------------------------
#include "colourvaluepref.moc"

