/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#include "doscolourbox.h"
#include "labelini.h"
//-------------------------------------------------------------------
#include <qpainter.h>
//-------------------------------------------------------------------
#include <kcolordialog.h>
#include <kconfig.h>
//-------------------------------------------------------------------
DosColourBox::ColourList DosColourBox::colourList;
DosColourBox::DlgList    DosColourBox::dlgList;
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::DosColourBox
 */
/**
 * Widget Creator.
 * Addresses of instantiated objects are stored in a list to facilitate global changes.
 * @param parent Widget parent.
 * @param name Widget name.
 * @param li	Inifile enabled QLabel used to facilitate storage of settings
 */
DosColourBox::DosColourBox(QWidget    * parent, 
						   const char * name,
						   LabelIni   * li )
			: KColorCells(parent, 2, 8)
			, label (li) {
	setName ( name );
	dlgList.append ( this );
	
// If this was the first, then establist the base colour set
	if ( colourList.count () == 0 ) {
		KConfig * config = KGlobal::config ();
		KConfigGroup group ( config, "Colours" );
		QValueList<QColor>::iterator it;
		QString 	 qs;
		QColor		 qc;
		
		colourList 	<<	Qt::black
					<<  Qt::darkBlue
					<<  Qt::darkGreen
					<<  Qt::darkCyan
	 
					<<  Qt::darkRed
					<<  Qt::darkMagenta
					<<  Qt::darkYellow
					<<  Qt::gray
	 
					<<	Qt::darkGray
					<<	Qt::blue
					<<	Qt::green
					<<	Qt::cyan
	 
					<<	Qt::red
					<<	Qt::magenta
					<<	Qt::yellow
					<<	Qt::white;

// Here, we need to load the adjusted colours
// Colour_0 = 0x00ff00ff
		
		
		for ( unsigned int uiIndex = 0; uiIndex < colourList.count (); uiIndex++ ) {
			it = colourList.at ( uiIndex );
			qs.sprintf ( "Colour_%x", uiIndex );
			qc = *it;
			qc = group.readColorEntry ( qs, &qc );
			*it = qc;
		} /* endfor */
		
	} /* endif */
	
// Place those into the Colour Grid
	for ( unsigned int uiIndex = 0; uiIndex < colourList.count (); uiIndex++ ) {
		setColor ( uiIndex, colourList [uiIndex] );
	} /* endfor */
	
	setCellWidth  ( 74 );
	setCellHeight ( 74 );
	show ();
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::~DosColourBox
 */
/**
 * Destructor
 */
DosColourBox::~DosColourBox() {
	dlgList.remove ( this );
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::shortAndWide
*/
/**
 * Alters colourbox to 2 rows by 8 columns
 * @param  None
 */
void DosColourBox::shortAndWide ( void ) {
	setNumRows ( 2 );
	setNumCols ( 8 );
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::updateColourList
 */
/**
 * Alters colourbox to 4 rows by 4 columns
 * @param  None
 */
void DosColourBox::square ( void ) {
	setNumRows ( 4 );
	setNumCols ( 4 );
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::updateColourList
 */
/**
 *  Goes through all instantiated dialogs and updates their colours
 * @param  None
 */
void DosColourBox::updateColourList (void) {
	DosColourBox * obj;
	
	for ( obj = dlgList.first (); obj; obj = dlgList.next () ) {
		for ( unsigned int uiIndex = 0; uiIndex < colourList.count (); uiIndex++ ) {
			obj->setColor ( uiIndex, colourList [uiIndex] );
		} /* endfor */	
	} /* endfor */	
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::updateColour(QColor colour)
 */
/**
 * Updates a single colour in the list
 * @param uiIndex Integer
 * @param colour QColor
 * @return true on error (index out of bounds)
 */
bool DosColourBox::updateColour(unsigned int uiIndex, QColor colour) {
	if ( uiIndex >= colourList.count () ) return true;
    colourList [uiIndex] = colour;
	updateColourList ();
	return false;
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::updateAllColours (QValueList<QColor> list)
 */
/**
 * Updates all colours in the list
 * @param list QValueList containing 16 QColor items
 * @return True on error (incorrect colour count)
 */
bool DosColourBox::updateAllColours (QValueList<QColor> list) {
    if ( list.count () != colourList.count () ) return true;
	colourList = list;
	updateColourList ();
	return false;
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::setSelected(int item)
 */
/**
 * Sets the current selection to the desired index.
 * @param iIndex 
 */
void DosColourBox::setSelected(int iIndex) {
    selected = iIndex;
	// repaint ();
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::getColour
 */
/**
 * Retrieves the currently selected colour
 * @return QColor reference
 */
QColor& DosColourBox::getColour () {
	return colourList [selected];
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::minimumSize()
 */
/**
 * Retrieves the minimum size the widget can take
 * @return QSize
 */
QSize DosColourBox::minimumSize() {
    return sizeHint ();
}
//-------------------------------------------------------------------
// Config
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::read (KConfigBase * config )
 */
/**
 * Reads the selected colour index from the inifile.
 * @param config Address of KConfigGroup object
 * @param sKey	Prepended to lable name (with a dot seperator)
 */
void DosColourBox::read (KConfigGroup * config, QString sKey ) {
	int iIndex = label->read ( config, sKey );
	setSelected ( iIndex );
}
//-------------------------------------------------------------------
/*!
    \fn DosColourBox::write ( KConfigBase * config )
 */
/**
 * Writes the selected colour index to the inifile.
 * @param config Address of KConfigGroup object
 * @param sKey Prepended to lable name (with a dot seperator)
 */
void DosColourBox::write ( KConfigGroup * config, QString sKey ) {
	int iIndex = getSelected ();
	label->write ( config, 
				   iIndex, 
	   			   sKey );
}
//-------------------------------------------------------------------
