/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#include <qstring.h>
//-------------------------------------------------------------------
#include <kconfig.h>
//-------------------------------------------------------------------
#include "labelini.h"
//-------------------------------------------------------------------
// Default 'Auto' setting is hexadecimal (my code, my rules !)
unsigned int LabelIni::aBase [] = { 16, 2, 8, 10, 16 };
QStringList LabelIni::aslBase = QStringList::split ( ',', "Auto,Bin,Oct,Dec,Hex" );
//-------------------------------------------------------------------
LabelIni::LabelIni ( QWidget * 		parent, 
					 const char * 	name, 
					 WFlags 		f ) 
   	    : QLabel ( parent, name, f ) {
	base = Hex;
}
//-------------------------------------------------------------------
LabelIni::LabelIni ( const QString & text, 
					 QWidget * 		parent, 
					 const char * 	name, 
					 WFlags 		f )
	: QLabel ( text, parent, name, f ) {
	base = Hex;
}
//-------------------------------------------------------------------
LabelIni::LabelIni ( QWidget * 		 buddy, 
					 const QString & text, 
					 QWidget * 		 parent, 
					 const char * 	 name, 
					 WFlags 		 f )
	: QLabel ( buddy, text, parent, name, f ) {
	base = Hex;
}
//-------------------------------------------------------------------
LabelIni::~LabelIni () {
}
//-------------------------------------------------------------------
// Base functions
// 
// return true on error (not decimal, octal, binary or hexadecimal)
// Do not search 'auto' setting !
//-------------------------------------------------------------------
bool LabelIni::setBase ( unsigned int ui ) {
	for ( unsigned int uiIndex = 1; 
		  uiIndex < ( sizeof ( aBase )/sizeof ( *aBase) ); 
		  uiIndex++ ) {
		if ( ui != aBase [uiIndex] ) continue;
		base = (Base) uiIndex;		
		return false;
	} /* endfor */
	return true;
}
//-------------------------------------------------------------------
bool LabelIni::setDefaultBase ( unsigned int uiBase ) {
	for ( unsigned int uiIndex = 1; 
		  uiIndex < ( sizeof ( aBase ) / sizeof ( *aBase) ); 
		  uiIndex++ ) {
		if ( uiBase != aBase [uiIndex] ) continue;
		aBase [0] = uiBase;		
		return false;
	} /* endfor */
	return true;
}
//-------------------------------------------------------------------
bool LabelIni::setDefaultBase ( QString sBase ) {
	int iIndex = aslBase.findIndex ( sBase );
	
	if ( iIndex < 1 ) return true;	// Do not allow 'Auto' (!)
	aBase [0] = aBase [iIndex];
	return false;
}
//-------------------------------------------------------------------
// Save keyVal with a string representing the value
QString LabelIni::keyName ( QString sKey,
							QString sName ) {
	QStringList sl;
	QString 	sKeyName;

	if ( sName.isEmpty () ) 
		sName = text ();

	else
		setText ( sName );
		
// If still empty, exit
	if ( sName.isEmpty () ) {
		bFault = true;
		return 0;
	} /* endif */

	sl = QStringList::split ( ".", sKey );
	sl << sName;
	sKeyName = sl.join (".");
	return sKeyName;
}
//-------------------------------------------------------------------
// read/write
//
// if text in label not set, it will be set from name of value
// if name of value not supplied, it is taken from text
//-------------------------------------------------------------------
// Save keyVal with a string representing the value
int LabelIni::read ( KConfigGroup * config, 
					 QString   sKey, 
					 QString   sName ) {
	QString 	sValue;
	int 		iIndex;
		
	sValue = config->readEntry ( keyName (sKey, sName), "" );
	
// check for prefix
	if ( sValue.length () > 1 ) { 
		if ( sValue [0] != '0' ) {
			iIndex = sValue.toInt ( &bFault, 10 );	
			base = Dec;
			
		} else
			switch ( sValue [1].latin1 () ) {
			case 'x':
			case 'X':
				iIndex = sValue.toInt ( &bFault, 16 );	
				base = Hex;
				break;
				
			case 'b':
			case 'B':
				base = Bin;
				sValue.remove ( 0, 2 );	// remove characters
				iIndex = sValue.toInt ( &bFault, 2 );	
				base = Bin;
				break;
				
			default:
				iIndex = sValue.toInt ( &bFault, 8 );
				base = Oct;	
				break;		
			} /* endswitch */
	} else {
		iIndex = sValue.toInt ( &bFault, 10 );	
		base = Dec;
	} /* endif */
	
	return iIndex;
}
//-------------------------------------------------------------------
// Save keyVal with a string representing the value
void LabelIni::write ( 	KConfigGroup * config, 
						int		  iValue, 
						QString	  sKey, 
						QString	  sName ) {
	QString 	sValue;
	
// insert base identifier into string
	switch ( base ) {
	case Bin:
		sValue.sprintf ( "0b%s",
						 QString::number ( iValue, 2 ) );
		break;
				
	case Oct:
		sValue.sprintf ( "0%s",
						 QString::number ( iValue, 8 ) );
		break;
				
	case Dec:
		sValue.sprintf ( "%d", iValue );
		break;
				
	case Hex:
		sValue.sprintf ( "0x%x", iValue );
		break;
	} /* endswitch */
	
	config->writeEntry ( keyName (sKey, sName), sValue );
	return;
}
//-------------------------------------------------------------------
// Bool 
//
// "on", "yes", "1" and "true" are recognised as 'active'.
//-------------------------------------------------------------------
bool LabelIni::readBool ( KConfigGroup * config, 
						  QString   sKey, 
						  QString   sName ) {
	bool bValue;
							  
	bValue = config->readBoolEntry ( keyName (sKey, sName), "" );
	return bValue;
}
//-------------------------------------------------------------------
/*
	void writeEntry ( 	const QString& pKey, 
						bool bValue,
						bool bPersistent = true, 	// Write at deletion time
						bool bGlobal = false,		// write to KDE global file
						bool bNLS = false			// Tag with location ID
*/
void LabelIni::writeBool ( 	KConfigGroup * config, 
							bool 		bValue,
							QString	  sKey, 
							QString	  sName ) {
	config->writeEntry ( keyName (sKey, sName), bValue );
}
//-------------------------------------------------------------------
// conversion validity (for read)
//-------------------------------------------------------------------
bool LabelIni::isFault ( void ) {
	return bFault;
}
//-------------------------------------------------------------------


