/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#include "linestyle.h"
#include "labelini.h"
//-------------------------------------------------------------------
#include <qpen.h>
#include <qpainter.h>
#include <qlayout.h>
//-------------------------------------------------------------------
#include <kconfig.h>
//-------------------------------------------------------------------
extern QColor clScheditBg;
//-------------------------------------------------------------------
LineStyle::StyleList LineStyle::styleList;
//-------------------------------------------------------------------
LineStyle::LineStyle(QWidget *parent, const char *name, LabelIni * li)
	: QGridView(parent, name, 0)
	, label ( li ) {
	QSize size;
	
	if ( styleList.count () == 0 ) {
		styleList 	<<	Qt::SolidLine
					<<  Qt::DashLine
					<<  Qt::DotLine
					<<  Qt::DashDotLine
					<<  Qt::DashDotDotLine
					<<  Qt::NoPen;	
	} /* endif */
		
	setCellHeight ( 30 );
	setCellWidth  ( 74 );
	setVScrollBarMode ( QScrollView::AlwaysOff );
	setNumRows (1);
	setNumCols (6);
	pen = QPen ( Qt::black, 1 );
	setHScrollBarMode ( QScrollView::AlwaysOff );
	setVScrollBarMode ( QScrollView::AlwaysOff );
}
//-------------------------------------------------------------------
void LineStyle::paintCell ( QPainter * p, int row, int col ) {
	int iIndex = row * numCols () + col;
	int y;
	int cx = cellWidth ();
	int cy = cellHeight ();
	QPen cellPen ( pen );
	int iPenWidth;
	int iFrameStyle;
	
	iPenWidth = ( selected != iIndex ) ? 1 : 3;
	p->setPen ( QPen ( Qt::gray, iPenWidth ) );
	QRect rect ( 2, 2, cx - 2, cy - 2 );
	//p->drawRect ( rect );
	iPenWidth = ( selected != iIndex ) ? 1 : 3;
	
	setLineWidth ( 4 );
	setMidLineWidth ( 3 );
//	setFrameWidth ( 4 );
	iFrameStyle = QFrame::StyledPanel;
	if ( selected != iIndex ) 
		iFrameStyle |= QFrame::Raised;
	
	else
		iFrameStyle |= QFrame::Sunken;
		
	setFrameStyle( iFrameStyle  );
	
//	setEraseColor ( clScheditBg );
	drawFrame (p);

	y = rect.top () + ( cellHeight () >> 1 );
	QPoint qpStart = QPoint ( rect.left () + 4, y );
	QPoint qpEnd   = qpStart + QPoint ( rect.width () - 8, 0  );
	
	cellPen.setStyle ( styleList [iIndex] );
	p->setPen ( cellPen );
	p->drawLine ( qpStart, qpEnd );
}
//-------------------------------------------------------------------
// Index to row, col
//-------------------------------------------------------------------
/*!
    \fn LineStyle::index ( int iRow, int iCol )
 */
int LineStyle::index ( QCoord qc ) {
	int iIndex;
	int iColCount = numCols ();
	
	iIndex = ( qc.y () * iColCount ) + qc.x ();
	return iIndex;
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::coord ( int iIndex )
 */
QCoord LineStyle::coord ( int iIndex ) {
	QCoord qc;
	int iColCount = numCols ();
	
	qc.setX ( iIndex % iColCount );
	qc.setY ( iIndex / iColCount );
	return qc;
}
//-------------------------------------------------------------------
void LineStyle::updateCell ( int iIndex, bool bErase ) {
	QCoord qc;
	
	qc = coord ( iIndex );
	repaintCell ( qc.y (), qc.x (), bErase );
}
//-------------------------------------------------------------------
void LineStyle::setSelected ( QMouseEvent * e ) {
	QCoord qc;
	int    iIndex;
	
	qc.setY (rowAt    ( e->y () ) );
	qc.setX (columnAt ( e->x () ) );
	iIndex = index (qc);
	setSelected ( iIndex );
}
//-------------------------------------------------------------------
// get/set
//-------------------------------------------------------------------
/*!
    \fn LineStyle::setPen ( QPen pen )
 */
void LineStyle::setPen ( QPen _pen ) {
	pen = _pen;
	//repaint ();
	
	for ( int iCol = 0; iCol < numCols (); iCol++ )
		for ( int iRow = 0; iRow < numRows (); iRow++ )
			repaintCell ( iRow, iCol, true );
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::getPen ()
 */
QPen& LineStyle::getPen () {
    return pen;
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::mousePressEvent ( QMouseEvent * e )
 */
void LineStyle::mousePressEvent ( QMouseEvent * e ) {
	setSelected ( e );
	emit selectionChanged ( selected );
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::getSelected ()
 */
int LineStyle::getSelected () {
	return selected;
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::setSelected (int iIndex)
 */
void LineStyle::setSelected (int iIndex) {
	if ( selected != -1 ) {
		selected = -1;
		updateCell ( iIndex, true );
	} /* endif */
	
	selected = iIndex;
	updateCell ( iIndex, false );
	pen.setStyle ( styleList [selected] );
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::sizeHint ()
 */
QSize LineStyle::sizeHint () const {
	QSize size = QSize ( cellWidth  () * numCols () + 5, 
			     	     cellHeight () * numRows () + 10);
	return size;
}
//-------------------------------------------------------------------
// Config
//-------------------------------------------------------------------
/*!
    \fn LineStyle::read (KConfigBase * config, QString sKey )
 */
void LineStyle::read (KConfigGroup * config, QString sKey ) {
	int iIndex = label->read ( config, sKey );
	setSelected ( iIndex );
}
//-------------------------------------------------------------------
/*!
    \fn LineStyle::write ( KConfigBase * config, QString sKey )
 */
void LineStyle::write ( KConfigGroup * config, QString sKey ) {
	int iIndex = getSelected ();
	label->write ( config, 
				   iIndex, 
				   sKey );
}
//-------------------------------------------------------------------
void LineStyle::setupPainter (QPainter& painter) {
	painter.setPen ( pen );
	painter.setBrush ( clScheditBg );
}
//-------------------------------------------------------------------
#include "linestyle.moc"
