/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ScheditBitmap.h"
#include "asciil.h"
#include <qpainter.h>
//---------------------------------------------------------------------------
int PinRecord::iBorder      = 40;
int PinRecord::iRadius      = 3;
int PinRecord::iVertLength  = 6;
int PinRecord::iBaseWidth   = 6;
int PinRecord::iPinTextGap  = 2;
//---------------------------------------------------------------------------
//                  Structure 'PinRecord'
//---------------------------------------------------------------------------
PinRecord::PinRecord ( unsigned char * & puc,
					   ScheditBitmap * owner ) {
	char asciilPin [5];
	char asciilFun [7];
	
	sizeBmp = owner->getBmpSize ();
	iExtension = owner->getExtension ();
	
	ucSection = *puc;	
	puc += sizeof ( ucSection );
			
	memcpy ( &purpose, puc, sizeof ( PinPurpose ) );
	puc += sizeof ( PinPurpose );

	memcpy ( asciilPin, puc, sizeof ( asciilPin ) );
	puc += sizeof ( asciilPin );
	Asciil alPin ( asciilPin );
	qsPin = alPin.toQString ();

	memcpy ( asciilFun, puc, sizeof ( asciilFun ) );
	puc += sizeof ( asciilFun );
	Asciil alFun ( asciilFun );
	qsFun = alFun.toQString ();

	ucSide = (char) *puc;
	puc++;

	ucSpacing = (char) *puc;
	puc++;
}
//---------------------------------------------------------------------------
PinRecord::~PinRecord () {
	//
}
//---------------------------------------------------------------------------
// Draw wires, invert circles and clock triangles on transformed image
void PinRecord::RenderPin ( QPainter& painter ) {
    QPoint 		 pin;
    QPoint       tpBorder;
	QPoint       tpMoveBy ( 0, 0 );
    QPoint       tpInner;
	QPoint       qpBaseAdj;
    eScheditSide side;
	int          iBaseHalfWidth;

    if ( purpose.bHide ) return;
	pin = getPoint ();

// draw wire extensions ( length 11 or 31, depending on device type )
    side = (eScheditSide) ucSide;
    tpInner  = getOuterPoint ( pin, side );            // Moved out if invert
	
	tpBorder = pin;
    switch ( side ) {
    case eBorderLeft:
        tpMoveBy.setX ( iExtension * -1 );
        break;

    case eBorderBottom:
		tpMoveBy.setY ( iExtension );
        break;

    case eBorderRight:
        tpMoveBy.setX ( iExtension );
        break;

    case eBorderTop:
        tpMoveBy.setY ( iExtension * -1 );
        break;

    default:
        // Could there be a connection to the center ?
        break;
    } /* endswitch */
	tpBorder += tpMoveBy;

// Render line
	painter.drawLine ( tpInner, tpBorder );

// Render invert circle as required
    if ( purpose.bInvert ) {
        QPoint tpTL = pin;
        QPoint tpBR = getOuterPoint ( pin, side );

        switch ( side ) {
        case eBorderLeft:
			tpTL -= QPoint ( 1, 0 );
			tpBR -= QPoint ( 1, 0 );
			
        case eBorderRight:
            tpTL -= QPoint ( 0, iRadius );
            tpBR += QPoint ( 0, iRadius );
            break;

        case eBorderTop:
        case eBorderBottom:
            tpTL -= QPoint ( iRadius, 0 );
            tpBR += QPoint ( iRadius, 0 );
            break;
        } /* endswitch */
		painter.setBrush ( QBrush ( Qt::NoBrush ) );
		painter.drawEllipse ( QRect (tpTL, tpBR) );
    } /* endif */

// Render clock triangle as required
    if ( purpose.bClock ) {
		QPointArray pa (3);
		enum {
            eClockBaseA,
            eClockVertice,
            eClockBaseB
        };
		
		iBaseHalfWidth = iBaseWidth >> 1;
        switch ( ucSide ) {
        case eBorderLeft:
        case eBorderRight:
            qpBaseAdj = QPoint ( 0, iBaseHalfWidth );
            break;

        case eBorderBottom:
        case eBorderTop:
            qpBaseAdj = QPoint ( iBaseHalfWidth, 0 );
            break;
        } /* endswitch */
		pa.setPoint ( eClockBaseA, 	 pin - qpBaseAdj);
		pa.setPoint ( eClockVertice, getInnerPoint ( pin, side ) );
		pa.setPoint ( eClockBaseB, 	 pin + qpBaseAdj);
        painter.drawPolyline ( pa );
    } /* endif */
}
//---------------------------------------------------------------------------
// Data from pin records
QPoint PinRecord::getPoint ( void ) {
    QPoint tpRc ( 0, 0 );

// Set border data
    switch ( ucSide ) {
    case eBorderLeft:
    case eBorderTop:
        break;

    case eBorderBottom:
        tpRc.setY ( sizeBmp.height () - 1 );
        break;

    case eBorderRight:
		tpRc.setX ( sizeBmp.width () - 1 );
        break;

	default:
        break;
    } /* endswitch */

// set offset data
    switch ( ucSide ) {
    case eBorderLeft:
    case eBorderRight:
        tpRc.setY ( ((int) ucSpacing) * 10 );
        break;

    case eBorderBottom:
    case eBorderTop:
        tpRc.setX ( ((int) ucSpacing) * 10 );
        break;

    default:
        tpRc = QPoint ( -1, -1 );
        break;
    } /* endswitch */

    return tpRc;
}
//---------------------------------------------------------------------------
// Moves point out from centre (+ve) or into centre (-ve) as required
void PinRecord::expandPoint ( QPoint&      pin,
							  eScheditSide side,
                              int          iExpand ) {
    switch ( side ) {
    case eBorderLeft:
        pin -= QPoint ( iExpand, 0 );
        break;

    case eBorderRight:
        pin += QPoint ( iExpand, 0 );
        break;

    case eBorderBottom:
        pin += QPoint ( 0, iExpand );
        break;

    case eBorderTop:
        pin -= QPoint ( 0, iExpand );
        break;
    } /* endswitch */
}
//---------------------------------------------------------------------------
// Pin position, or point inside clock triangle if present
// For drawing pin strings
QPoint PinRecord::getInnerPoint ( QPoint pin, eScheditSide side ) {
    if ( purpose.bClock ) 
		expandPoint ( pin, side, iVertLength * -1 );
	
    return pin;
}
//---------------------------------------------------------------------------
// Pin position, or point outside invert circle if present
// For drawing pin lines & numbers
QPoint PinRecord::getOuterPoint ( QPoint pin, eScheditSide side ) {
    if ( purpose.bInvert )
        expandPoint ( pin, side, iRadius * 2 );

    return pin;
}
//---------------------------------------------------------------------------
// debugging
void PinRecord::drawCrosses ( QPainter& painter, 
							  QPoint qp ) {
	int iCross = 3;
	QPoint qpOffset ( iCross, iCross );

	QPen oldPen = painter.pen ();
	painter.setPen ( QPen ( Qt::magenta, 2 ) );
	painter.drawLine ( qp - qpOffset,
					   qp + qpOffset );
	
	qpOffset = QPoint ( iCross, iCross * -1 );
	painter.drawLine ( qp - qpOffset,
					   qp + qpOffset );
	painter.setPen ( oldPen );
}
//---------------------------------------------------------------------------
// Pin numbers are not rotated. Called from 'Untransformed'
void PinRecord::PinNum ( QPainter&    painter,
						 QPoint       pin,
						 eScheditSide side ) {
    QPoint              tpText;
	QRect 				rectText; 
	QFontMetrics fm ( ScheditBitmap::getNumFontMetrics () );
	
    if ( purpose.bHide ) return;
    if ( qsPin.isEmpty () ) return;
    tpText = getOuterPoint ( pin, side );
//	drawCrosses ( painter, tpText );
	rectText = fm.boundingRect ( qsPin );
    switch ( side ) {
    case eBorderLeft:
		tpText -= QPoint ( rectText.width () + iPinTextGap, 
						   0 - rectText.height () - iPinTextGap );
		break;
                                    // intentional fall-through
    case eBorderRight:
		tpText += QPoint ( iPinTextGap, 
						   rectText.height () + iPinTextGap );
        break;

    case eBorderTop:
		tpText -= QPoint ( iPinTextGap * -1, 
						   rectText.height () + iPinTextGap );
//...
// intentional fall-through
//...
    case eBorderBottom:
        tpText += QPoint ( iPinTextGap, 
						   rectText.height () + iPinTextGap ); // moves text left of pin line
        break;
    } /* endswitch */
	painter.drawText (  tpText, qsPin );
}
//---------------------------------------------------------------------------
// Pin strings are rotated 90 degrees for top and bottom pins only
// So, for those sides, the canvas is presented to this method 
// having been rotated by 90 deg clockwise. We then draw our text,
// and then rotate the result back 90 deg anticlockwise.
void PinRecord::PinStr ( QPainter&    painter,
						 QPoint       pin,
						 eScheditSide side ) {
	QFontMetrics fm ( ScheditBitmap::getFunFontMetrics () );
	QPoint		qpText;
	QRect 		rectText; 
	QWMatrix	wm ( 0, -1, -1, 0, 0, sizeBmp.width () );

    if ( purpose.bHide ) return;
    if ( qsFun.isEmpty() ) return;
    qpText = getInnerPoint ( pin, side );
	
// rotate point if required
	rectText = fm.boundingRect ( qsFun );
	switch ( side ) {
	case eBorderBottom:
		qpText = wm.map ( qpText );
		side   = eBorderLeft;
		break;
			
	case eBorderTop:
		qpText = wm.map ( qpText );
		side   = eBorderRight;
		qpText -= QPoint ( rectText.height (), 0 );
		break;
			
	case eBorderRight:
		qpText -= QPoint ( rectText.width (), 0 );
		break;
		
	default:
		break;
	} /* endswitch */
	
	expandPoint ( qpText, side, iPinTextGap * -1 );   // Move text in
	
// Drop one-half of character height for 
	int iHalf = rectText.height () >> 1;
	qpText += QPoint ( 0, iHalf );
	painter.drawText ( qpText, qsFun );
}
//---------------------------------------------------------------------------
QString PinRecord::Export ( QStringList * slFunction ) {
    QString asSide = "LBRT";
    QString as;
	QString qsFunction;
	QString qsPinName;
	QString qsFunctionName;
	QChar  cSide;

	qsPinName      = qsPin.isEmpty () ? "" : qsPin.ascii ();
	qsFunctionName = qsFun.isEmpty () ? "" : qsFun.ascii ();
	qsFunction = *slFunction->at (purpose.ucMainFunc);
	cSide = asSide.at (ucSide);
    as.sprintf ( "%c%d   %s    '%s'         %s",
                 cSide.latin1 (),
                 ucSpacing,
                 qsPinName.ascii (),
                 qsFunctionName.ascii (),
                 qsFunction.ascii () );

    return as;
}
//---------------------------------------------------------------------------
PinRecord::eScheditSide PinRecord::getSide ( void ) {
	return (eScheditSide) ucSide;
}
//---------------------------------------------------------------------------
bool PinRecord::sectionMismatch ( int iSection ) {
	return ( (int) ucSection != iSection );
}
//---------------------------------------------------------------------------

