/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PINRECORD_H
#define PINRECORD_H

#include <qstring.h>
#include <qpoint.h>
#include <qbitmap.h>

class QCanvas;
class ScheditBitmap;

/**
	@author James Fuller,,, <james@slakware12>
*/
class PinRecord{
public:
    typedef enum {
        eBorderLeft,
        eBorderBottom,
        eBorderRight,
        eBorderTop
    } eScheditSide;

#pragma pack (push,1)
private:
    typedef struct {
        unsigned char bInvert:1;
        unsigned char bClock:1;
        unsigned char bHide:1;
        unsigned char bBit3:1;
        unsigned char ucMainFunc:4;
    } PinPurpose;
#pragma pack (pop)

	QString qsPin;
	QString qsFun;
	QSize   sizeBmp;
	
	unsigned char ucSection;
	PinPurpose    purpose;

	unsigned char ucQueA   [2];
	unsigned char ucSide;       // { Left =0, Bottom, Right, Top }
	unsigned char ucSpacing;    // multiply by ten

	int iExtension;
	
    static int iBorder;
    static int iRadius;
    static int iVertLength;
    static int iBaseWidth;
    static int iPinTextGap;

    void   expandPoint   ( QPoint&      tp,
						   eScheditSide side,
                           int          iExpand );
    QPoint getInnerPoint ( QPoint pin, eScheditSide side );   // inside clock triangle
    QPoint getOuterPoint ( QPoint pin, eScheditSide side );   // outside invert circle
    QString Export ( QStringList * slFunction );
	void drawCrosses ( QPainter& painter, QPoint qp );		// Debugging

public:
	PinRecord ( unsigned char * & puc,
			    ScheditBitmap *   Owner);

    ~PinRecord();

    void PinNum ( QPainter&    painter,
				  QPoint       point,
				  eScheditSide side );

    void PinStr ( QPainter&    painter,
				  QPoint       point,
				  eScheditSide side );

    void   RenderPin  ( QPainter& painter );

    QPoint getPoint      ( void );   // pin connection

	eScheditSide getSide ( void );
	
	bool sectionMismatch ( int iSection );

/*	static QFont fontPinStr ( void );
	static void setFontPinStr ( QFont font );
	
	static QFont fontFunStr ( void );
	static void setFontFunStr ( QFont font );*/
};

#endif
