/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include "scheditdocbox.h"
#include "colourvaluepref.h"
//---------------------------------------------------------------------------
#include <qfontmetrics.h>
#include <qpainter.h>
#include <kconfigbase.h>
//---------------------------------------------------------------------------
ScheditFontPrefs * ScheditDocBox::lpFont;
ColourValuePref  * ScheditDocBox::lpLine;
//---------------------------------------------------------------------------
ScheditDocBox::ScheditDocBox ( QCanvas * canvas, 
							   unsigned char * & puc )
 			: QCanvasRectangle ( canvas ) {
	int             iStart;
	int 			iIndex;
	unsigned char 	uc;
	QString      	as;
		
// header
	iLength = sizeof ( DocBoxHdr );
	memcpy ( &header, puc, iLength );
	puc += iLength;

// text
	iLength = header.ucLines << 4;
	pText = new char [iLength];
	memcpy ( pText, puc, iLength );
	puc += iLength;

	sl = new QStringList;
	for ( iIndex = 0, iStart = 0;
		 iIndex < iLength;
		 iIndex++ ) {
			 uc = pText [iIndex];
			 switch ( uc ) {
			 case 0x0d:
			 case 0x8d:
				 pText [iIndex] = 0;
				 sl->append ( (char *) &pText [iStart] );
				 break;

			 case 0x0a:
				 iStart = iIndex + 1;
				 break;

			 default:
				 break;
		 }
	}		 
	
// Location
	setSize ( header.usEndX - header.usStartX + 1,
			  header.usEndY - header.usStartY + 1 );
	setX ( header.usStartX );
	setY ( header.usStartY );
	
	show ();
}
//---------------------------------------------------------------------------
//ScheditDocBox::~ScheditDocBox()
//{
//}
//---------------------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int ScheditDocBox::rtti () const {
	return 1009;
}
//---------------------------------------------------------------------------
void ScheditDocBox::draw ( QPainter& painter ) {
	QStringList::Iterator it;
	QString				  qs;
	int 				  iY;
	QFontMetrics 		  fm ( lpFont->getFontMetrics () ); 
	
	painter.setBrush ( Qt::NoBrush );
	lpLine->setupPainter ( painter );
	drawShape (painter);
	
	// go through the lines and draw the text
	lpFont->setupPainter ( painter );
	iY = y () + 4;
	for ( it = sl->begin (); it != sl->end (); ++it ) {
		qs = *it;
		iY += fm.boundingRect ( qs ).height ();
		painter.drawText ( x () + 4, iY, qs );
	} /* endfor */
}
//---------------------------------------------------------------------------
// Preference Pages
//---------------------------------------------------------------------------
void ScheditDocBox::readProperties ( KConfig * config ) {	
	lpFont->readGroup ( config );
	lpLine->readGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditDocBox::writeProperties ( KConfig * config ) {	
	lpFont->writeGroup ( config );
	lpLine->writeGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditDocBox::setup ( KDialogBase * dlg ) {	
	lpLine = new ColourValuePref (dlg, 
								   "DocBox Line", 
		   						   "DocBox.Line" ); 
	lpLine->adjustForCheckBoxes ();	
	
	lpFont = new ScheditFontPrefs ( dlg, 
									"DocBox Text", 
									"DocBox.Text"); 
	
// What do we load into the canvas ?
	QCanvas * canvas = lpLine->getCanvas ();
	unsigned char puc [] = {
		0x50, 0x10, 0x00, 0x10, 0x00, 0x50, 0x01, 0x6a,
		0x00, 0x28, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00,
		0x43, 0x48, 0x41, 0x4e, 0x47, 0x45, 0x20, 0x4c,
		0x49, 0x53, 0x54, 0x3a, 0x0d, 0x0a, 0x2a, 0x72,
		0x65, 0x76, 0x65, 0x72, 0x73, 0x65, 0x20, 0x6c,
		0x6f, 0x67, 0x69, 0x63, 0x20, 0x6f, 0x75, 0x74,
		0x70, 0x75, 0x74, 0x20, 0x70, 0x69, 0x6e, 0x32,
		0x20, 0x66, 0x6f, 0x72, 0x20, 0x70, 0x6f, 0x73,
		0x73, 0x69, 0x62, 0x6c, 0x65, 0x8d, 0x0a, 0x61,
		0x63, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x69,
		0x6f, 0x64, 0x65, 0x73, 0x0d, 0x0a, 0x2a, 0x70,
		0x61, 0x64, 0x73, 0x20, 0x66, 0x6f, 0x72, 0x20,
		0x62, 0x75, 0x66, 0x66, 0x65, 0x72, 0x20, 0x74,
		0x72, 0x61, 0x6e, 0x73, 0x69, 0x73, 0x74, 0x6f,
		0x72, 0x73, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x74,
		0x68, 0x69, 0x73, 0x0d, 0x0a, 0x2a, 0x20, 0x70,
		0x61, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x62,
		0x69, 0x64, 0x69, 0x20, 0x68, 0x69, 0x67, 0x68,
		0x20, 0x75, 0x67, 0x72, 0x69, 0x64, 0x20, 0x6f,
		0x70, 0x74, 0x6f, 0x0d, 0x0a, 0x2a, 0x20, 0x31,
		0x32, 0x76, 0x20, 0x64, 0x69, 0x66, 0x66, 0x20,
		0x61, 0x6d, 0x70, 0x20, 0x72, 0x65, 0x76, 0x65,
		0x72, 0x73, 0x65, 0x20, 0x6f, 0x75, 0x74, 0x70,
		0x75, 0x74, 0x0d, 0x0a, 0x2a, 0x20, 0x6f, 0x70,
		0x74, 0x6f, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x74,
		0x68, 0x69, 0x73, 0x0d, 0x0a, 0x2a, 0x20, 0x70,
		0x61, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x62,
		0x69, 0x6e, 0x61, 0x72, 0x79, 0x20, 0x77, 0x65,
		0x69, 0x67, 0x68, 0x74, 0x65, 0x64, 0x20, 0x44,
		0x49, 0x50, 0x20, 0x73, 0x77, 0x69, 0x74, 0x63,
		0x68, 0x0d, 0x0a, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
		0x2e, 0x2e, 0x73, 0x65, 0x65, 0x20, 0x44, 0x4f,
		0x57, 0x4e, 0x32, 0x30, 0x30, 0x30, 0x2e, 0x50,
		0x43, 0x42, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e, 0x2e,
		0x2e, 0x8d, 0x0a, 0x0d, 0x0a, 0x20, 0x20, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
	};
	unsigned char * puc2 = (unsigned char *) puc;
	new ScheditDocBox ( canvas, puc2 );
}
//---------------------------------------------------------------------------



