/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License qs published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qrect.h>
#include <qsize.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qbuffer.h>
#include <qdir.h>
#include <qfileinfo.h>
//---------------------------------------------------------------------------
#include "scheditpagedef.h"
//---------------------------------------------------------------------------
#include "scheditfile.h"
//---------------------------------------------------------------------------
#include "scheditheader.h"
#include "scheditline.h"
#include "ScheditBitmap.h"
#include "scheditannotate.h"
#include "scheditdocbox.h"
#include "scheditjunction.h"
#include "scheditmod0x60.h"
//---------------------------------------------------------------------------
class QCanvasItem;
//---------------------------------------------------------------------------
ScheditFile::ScheditFile ( QCanvas * _canvas ) {
	canvas = _canvas;
	bExtract = false;
}
//---------------------------------------------------------------------------
ScheditFile::~ScheditFile()
{
}
//---------------------------------------------------------------------------
void ScheditFile::loadFromFile ( QString qsFileName ) {
	QString 		qsLastName;
	QString 		qs;
	QString 		qsPath;
	int 			iLength;
	unsigned char * puc;
	unsigned char * pucStart;
	unsigned char * pucLast;
	unsigned char 	uc;
	QByteArray      baDataFile;
	QSize           qsPage;

	QDataStream * fsBitmap;
	QDataStream * fsJunction;
	QDataStream * fsDocBox;
	QDataStream * fsLine;
	QDataStream * fsAnnotate;
	QCanvasItemList cil;
	QCanvasItemList::Iterator it;

	QFile file ( qsFileName );
	QFileInfo fileInfo ( file );
	
// Delete all the old stuff
	cil = canvas->allItems ();
	for ( it = cil.begin (); it != cil.end (); it++ ) {
		so = *it;
		so->setVisible (false);
		delete so;
	}
	
// Check file exists
	if ( ! file.exists () ) {
		qs.sprintf ( "File '%s' not found", qsFileName.ascii () );
		QMessageBox::warning ( NULL, 
							   "Application name",
							   "Unable to find the user file");
		return;
	} /* endif */

	file.close ();

// record new details	
	qfi.setFile (file);
	file.open ( IO_ReadOnly );
	baDataFile = file.readAll ();
	
	//	Caption = ExtractFileName ( qsFileName );

// Read data from File
	puc = (unsigned char *) baDataFile.data ();
	iLength = baDataFile.size ();
	pucLast = puc + iLength;

    rectPage = QRect ( 0, 0, canvas->width (), canvas->height () );
	if ( bExtract ) {
		QDir dir;
		
		qsExportPath.sprintf ( "%s/%s",
							   fileInfo.dirPath ().ascii (),
							   fileInfo.baseName ().ascii () );
		dir.setPath ( qsExportPath );
		if ( ! dir.exists () )
			dir.mkdir ( qsExportPath, true );

		fsBitmap   = MakeExportFile ( "Bitmap" );
		fsJunction = MakeExportFile ( "Junction" );
		fsDocBox   = MakeExportFile ( "DocBox" );
		fsLine     = MakeExportFile ( "Line" );
		fsAnnotate = MakeExportFile ( "Annotate" );
	} /* endif */

	header = NULL;	
	while ( puc < pucLast ) {
		uc = *puc;
		pucStart = puc;
		switch (uc) {
		case eHeader:
			so = (QCanvasItem *) new ScheditHeader ( canvas, puc );
			if ( header != NULL ) {
				qs.sprintf ( "Extraneous header '%02x' at offset 0x'%p'", 
							 *puc, 
							 (void *) (puc - (unsigned char *) baDataFile.data () ) );
				QMessageBox::warning ( NULL, 
									   "Application name",
										qs );
				break;
			} /* endif */
			header = (ScheditHeader *) so;
			header->setPage (); 
			qsPage = header->getSize ();
			canvas->resize ( qsPage.width (), qsPage.height () );
			header->BackgroundColour = Qt::white; 		//Color;
			header->setFileData ( qfi );
			break;

		case eLine:
			so = (QCanvasItem *) new ScheditLine ( canvas, puc );
			break;

		case eAnnotate:
			so = (QCanvasItem *) new ScheditAnnotate ( canvas, puc );
			break;

		case 0x40:
			qs.sprintf ( "Unknown header '0x40' at offset 0x'%p'", 
						 (void *) (puc - (unsigned char *) baDataFile.data () ) );
			QMessageBox::warning ( NULL, 
								   "Application name",
	   							   qs );
			puc += 0x10;
			continue;

		case eDocBox:
			so = (QCanvasItem *) new ScheditDocBox ( canvas, puc );
			break;

		case eJunction:
			so = (QCanvasItem *) new ScheditJunction ( canvas, puc );
			break;

		case 0x60:
			so = (QCanvasItem *) new ScheditMod0x60 ( canvas, puc );
			break;

		case eBitmap:
			so = (QCanvasItem *) new ScheditBitmap ( canvas, puc );
			break;

		default:
			qs.sprintf ( "Unknown header '%02x' at offset 0x'%p'",
						 *puc, 
						 (void *) (puc - (unsigned char *) baDataFile.data () ) );
			QMessageBox::warning ( NULL, 
								   "Application name",
	   							   qs );
			return;
		} /* endswitch */

		if ( !bExtract ) continue;
// Create Extracts if required
		iLength = puc - pucStart;
		switch (uc) {
		case eHeader:
			fsBitmap->writeRawBytes   ( (const char *) pucStart, iLength );
			fsJunction->writeRawBytes ( (const char *) pucStart, iLength );
			fsDocBox->writeRawBytes   ( (const char *) pucStart, iLength );
			fsLine->writeRawBytes     ( (const char *) pucStart, iLength );
			fsAnnotate->writeRawBytes ( (const char *) pucStart, iLength );
			break;

		case eBitmap:
			fsBitmap->writeRawBytes ( (const char *) pucStart, iLength );
			break;

		case eJunction:
			fsJunction->writeRawBytes ( (const char *) pucStart, iLength );
			break;

		case eDocBox:
			fsDocBox->writeRawBytes ( (const char *) pucStart, iLength );
			break;

		case eLine:
			fsLine->writeRawBytes ( (const char *) pucStart, iLength );
			break;

		case eAnnotate:
			fsAnnotate->writeRawBytes ( (const char *) pucStart, iLength );
			break;

		default:
			break;
		} /* endselect */
	} /* endwhile */

	if ( bExtract ) {
		delete fsBitmap;
		delete fsJunction;
		delete fsDocBox;
		delete fsLine;
		delete fsAnnotate;
	} /* endif */

//	delete baFileData;          // release file storage
	
//	Application->Title = Caption;
}
//---------------------------------------------------------------------------
QDataStream * ScheditFile::MakeExportFile ( QString qs ) {
	QString 		qsFile;
	QDataStream * 	fs;
	bool 			bFileExists;

	qsFile.sprintf ( "%s/%s.s01",
					 qsExportPath.ascii (),
   					 qs.ascii () );
	QFile file ( qsFile );
	
	bFileExists = file.exists ();
	fs = new QDataStream ( &file );
	return fs;
}
//---------------------------------------------------------------------------
void ScheditFile::setPage ( int iPageEnum ) {
	QSize ts;

	ScheditPageDef * pSpd = ScheditPageDef::find ( iPageEnum );
	ts = pSpd->getSize ();
	canvas->resize ( ts.width (), ts.height () ); 
}
//---------------------------------------------------------------------------
void ScheditFile::saveProperties ( KConfig * config ) {
	ScheditHeader::writeProperties   ( config );
	ScheditLine::writeProperties     ( config );
	ScheditJunction::writeProperties ( config );
	ScheditAnnotate::writeProperties ( config );
	ScheditDocBox::writeProperties   ( config );
	ScheditBitmap::writeProperties   ( config );
}
//---------------------------------------------------------------------------
void ScheditFile::readProperties ( KConfig * config ) {
	ScheditHeader::readProperties   ( config );
	ScheditLine::readProperties     ( config );
	ScheditJunction::readProperties ( config );
	ScheditAnnotate::readProperties ( config );
	ScheditDocBox::readProperties   ( config );
	ScheditBitmap::readProperties   ( config );
}
//---------------------------------------------------------------------------
void ScheditFile::setup ( KDialogBase * dlg ) {	
	ScheditHeader::setup   ( dlg );
	ScheditLine::setup     ( dlg );
	ScheditJunction::setup ( dlg );
	ScheditAnnotate::setup ( dlg );
	ScheditDocBox::setup   ( dlg );
	ScheditBitmap::setup   ( dlg );
}
//---------------------------------------------------------------------------


