/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "scheditheader.h"
//---------------------------------------------------------------------------
#include <qpainter.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcheckbox.h>
//---------------------------------------------------------------------------
#include <kconfig.h>
//---------------------------------------------------------------------------
#include "pointhelper.h"
#include "scheditlocatedtext.h"
#include "schedittitleblock.h"
#include "colourvaluepref.h"
//---------------------------------------------------------------------------
// Static Members
//---------------------------------------------------------------------------
ScheditLinePrefs * ScheditHeader::lpBackground;
ColourValuePref  * ScheditHeader::lpLine;
ScheditFontPrefs * ScheditHeader::lpText;

LabelIni  * ScheditHeader::laVisible     = NULL;
QCheckBox * ScheditHeader::cbShowBorders = NULL;
//---------------------------------------------------------------------------
QColor clScheditBg;
//-------------------------------------------------------------------
// Class ScheditHeader
//---------------------------------------------------------------------------
ScheditHeader::ScheditHeader ( QCanvas * canvas,
							   unsigned char * & puc )
			  : QCanvasItem  ( 			 canvas ) {
	QPoint tpTL ( 0, 0 );
	QString as;
	int iLength;
	int iPage;
	ScheditPageDef * pSpd;
	
	pSpd = NULL;

	iLength = sizeof ( hdr );
	memcpy ( &header, puc, iLength );
	puc += iLength;

	pTitleBlock = new ScheditTitleBlock ( canvas );
	
// load the appropriate diagram size
	iPage = (int) header.cPageEnum;
	setPageFromEnum ( (ScheditPageDef::ePageSize) iPage );
	show ();
}
//---------------------------------------------------------------------------
ScheditHeader::~ScheditHeader () {
}
//---------------------------------------------------------------------------
// We do not want to detect collisions
//---------------------------------------------------------------------------
bool ScheditHeader::collidesWith ( const QCanvasItem * other ) const {
	return false;
}
//---------------------------------------------------------------------------
bool ScheditHeader::collidesWith( const QCanvasSprite* cs,
								  const QCanvasPolygonalItem* pi,
								  const QCanvasRectangle* cr,
							      const QCanvasEllipse* ce,
								  const QCanvasText* ct) const {
	 return false;
}
//---------------------------------------------------------------------------
QRect ScheditHeader::boundingRect ( void ) const {
	return rectPage;
}
//---------------------------------------------------------------------------
void ScheditHeader::draw ( QPainter& painter ) {
	QString         		qs;
	QPointArray::Iterator	it;
	int						iIndex;
	bool					bShowBorders;

// Lines
	lpLine->setupPainter ( painter );
	
// Draw border
	canvas ()->setBackgroundColor ( clScheditBg );
	painter.drawRect ( rectPage  );
	
	bShowBorders = cbShowBorders->isChecked ();
	if ( ! bShowBorders ) return;
	
	painter.drawRect ( rectInner );

// Draw dividers in border
	painter.drawLineSegments ( paHor );
	painter.drawLineSegments ( paVer );
	
// Text
	lpText->setupPainter ( painter );
	
// Draw Horizontal text 
	iIndex = '1';
	for ( it = paHorText.begin(); it != paHorText.end(); ++it ) {
		qs.sprintf ( "%c", iIndex++ );
		painter.drawText ( (*it), qs );
		++it;
		painter.drawText ( (*it), qs );
	} /* endif */
	
// Draw Vertical text 
	iIndex = 'A';
	for ( it = paVerText.begin(); it != paVerText.end(); ++it ) {
		qs.sprintf ( "%c", iIndex++ );
		painter.drawText ( (*it), qs );
		++it;
		painter.drawText ( (*it), qs );
	} /* endif */
}
//---------------------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int ScheditHeader::rtti () const {
	return 1004;
}
//---------------------------------------------------------------------------
QRect ScheditHeader::getRect (bool& bDraw) {
	bDraw = true;
	return rectPage;
}
//---------------------------------------------------------------------------
QSize ScheditHeader::getSize ( void ) {
	return pSpd->getSize ();
}
//---------------------------------------------------------------------------
QSize ScheditHeader::getSize ( ScheditPageDef::ePageSize ps ) {
	ScheditPageDef * pSpd;
	
	pSpd = ScheditPageDef::find ( ps );
	return pSpd->getSize ();
}
//---------------------------------------------------------------------------
void ScheditHeader::setPage ( void ) {
	pSpd = ScheditPageDef::find ( header.cPageEnum );
}
//---------------------------------------------------------------------------
void ScheditHeader::setFileData ( QFileInfo qfi ) {
	if ( pTitleBlock ) pTitleBlock->setFileData ( qfi );
}
//---------------------------------------------------------------------------
QSize ScheditHeader::setPageFromEnum ( ScheditPageDef::ePageSize ps ) {
	int 	dX;
	int 	dY;
	int 	iSec;
	int 	x;
	int 	y;
	QRect 	rectText;
	QSize 	size;
	QString qs;
	QPoint  qpText;
	int 	iShifted;
	int		iBorderThickness = 20;
	int 	iCount;
	QPointArray pa;
	QFontMetrics fm ( lpText->getFontMetrics () );
	
// get page definition
	pSpd = ScheditPageDef::find ( (int) ps );
	if ( pSpd == NULL ) pSpd = ScheditPageDef::defPage ();		// safety
	if ( pTitleBlock ) pTitleBlock->setPageSize ( pSpd->getName () );
	
// Create rectangles for the diagram
	size = pSpd->getSize ();
	rectPage  = pSpd->getRect ();
	rectInner = QRect ( iBorderThickness, 			  
						iBorderThickness, 
						size.width  () - (iBorderThickness << 1), 
						size.height () - (iBorderThickness << 1) );
	
// Build geometry here
// draw reference boxes in horizontal plane
	paHor.resize ( 0 );						// Empty it
	paVer.resize ( 0 );
	paHorText.resize ( 0 );
	paVerText.resize ( 0 );
	
// Horizontal
	qs.sprintf ( "8" );						// Just a typical character
	rectText = fm.boundingRect ( qs ); 
	iShifted = iBorderThickness - rectText.height ();
	iShifted >>= 1;
	
	iCount = pSpd->getHdiv ();
	dX = size.width () / (iCount << 1);
	for ( iSec = 1, x = dX << 1; 
		  iSec < iCount; 
		  iSec++, x = iSec * dX * 2 ) {
			  
// Make a divider for the top row at this 'x' position
		pa.resize ( 0 );
		pa << QPoint ( x, 0 )
		   << QPoint ( x, iBorderThickness );
		
// Add that segment, translate it to the bottom row, and add again		
		paHor += pa;
		pa.translate ( 0, size.height () - iBorderThickness );
		paHor += pa;

// Add the location of the top section identifier on the top row
		qpText = QPoint ( x - dX, iBorderThickness - iShifted );
		paHorText << qpText;
// Translate it to the bottom row and add again
		qpText += QPoint ( 0, size.height () - iBorderThickness );
		paHorText << qpText;
	} /* endfor */
// Add the location of the top section identifier on the top row
	qpText = QPoint ( x - dX, iBorderThickness - iShifted );
	paHorText << qpText;
// Translate it to the bottom row and add again
	qpText += QPoint ( 0, size.height () - iBorderThickness );
	paHorText << qpText;
		
// Vertical
	qs.sprintf ( "A" );
	rectText = fm.boundingRect ( qs ); 
	iShifted = iBorderThickness - rectText.width ();
	iShifted >>= 1;
	
	iCount = pSpd->getVdiv ();
	dY = size.height () / (iCount << 1);
	for ( iSec = 1, y = dY << 1; 
		  iSec < iCount; 
		  iSec++, y = iSec * dY * 2 ) {
// Make a divider for the left column at this 'y' position
		pa.resize ( 0 );
		pa 	<< QPoint ( 0, 				  y )
			<< QPoint ( iBorderThickness, y );
		
// Add that segment, translate it to the right column, and add again		
		paVer += pa;
		pa.translate ( size.width () - iBorderThickness, 0 );
		paVer += pa;

// Add the location of the top section identifier on the top row
		qpText = QPoint ( iShifted, y - dY );
		paVerText << qpText;
// Translate it to the bottom row and add again
		qpText += QPoint ( size.width () - iBorderThickness, 0 );
		paVerText << qpText;
	} /* endfor */
// Add the location of the top section identifier on the top row
	qpText = QPoint ( iShifted, y - dY );
	paVerText << qpText;
// Translate it to the bottom row and add again
	qpText += QPoint ( size.width () - iBorderThickness, 0 );
	paVerText << qpText;
	
// Create translation for TitleBox
	pTitleBlock->setOffset ( rectInner.bottomRight () );
	return size;
}
//---------------------------------------------------------------------------
// Private class PageDef
//---------------------------------------------------------------------------
QSize ScheditHeader::PageDef::getSize ( void ) {
	return page;
}
//---------------------------------------------------------------------------
QRect ScheditHeader::PageDef::getRect ( void ) {
	QRect rect ( 0, 0, page.width(), page.height () );
	return rect;
}
//---------------------------------------------------------------------------
int ScheditHeader::PageDef::getHdiv ( void ) {
	return hDiv;
}
//---------------------------------------------------------------------------
int ScheditHeader::PageDef::getVdiv ( void ) {
	return vDiv - '@';
}
//---------------------------------------------------------------------------
// Preferences
//---------------------------------------------------------------------------
void ScheditHeader::setup ( KDialogBase * dlg ) {	
	QString qsRoot = "Page";
										 
	lpBackground = new ScheditLinePrefs (dlg, 
										 qsRoot + QString (" Background"),
										 "Page.Background" ); 
	lpBackground->hideStyle();
	lpBackground->hideValue ();
	lpBackground->adjustSize ();
	
	qsRoot = "Cross Reference";
	lpLine = new ColourValuePref (dlg, 
								   qsRoot + QString (" Line"),
									"XRef.Line" ); 
	new QLabel ( lpLine );
	laVisible     = new LabelIni  ( "Visibility",  lpLine );
	cbShowBorders = lpLine->makeCb ( "Show Border" );
	lpLine->embolden ( laVisible );	
	lpLine->adjustForCheckBoxes ();	
	
	lpText = new ScheditFontPrefs (dlg, 
								   qsRoot + QString (" Text"),
								   "XRef.Text" ); 
	
	ScheditTitleBlock::setup ( dlg );
	
// Sample Header
	unsigned char puc [] = { 
		0x00, 0x01, 0x01, 0x00, 0x0b, 0x53, 0x43, 0x48, 
  		0x45, 0x4d, 0x41, 0x54, 0x49, 0x43, 0x20, 0x20
	};
	QCanvas * canvas = lpLine->getCanvas ();
	unsigned char * puc2 = puc;
	new ScheditHeader ( canvas, puc2 );
}
//---------------------------------------------------------------------------
void ScheditHeader::readProperties ( KConfig * config ) {
	lpBackground->readGroup ( config );
	lpLine->readGroup ( config );
	lpText->readGroup ( config );
	ScheditTitleBlock::readProperties ( config );
	
	// save background colour globally
	clScheditBg = lpBackground->getPen ().color ();
}
//---------------------------------------------------------------------------
void ScheditHeader::writeProperties ( KConfig * config ) {	
	lpBackground->writeGroup ( config );
	lpLine->writeGroup ( config );
	lpText->writeGroup ( config );
	ScheditTitleBlock::writeProperties ( config );
}
//---------------------------------------------------------------------------



