/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#ifndef SCHEDITHEADER_H
#define SCHEDITHEADER_H
//-------------------------------------------------------------------
#include <qcanvas.h>
#include <qrect.h>
#include <qpointarray.h>
#include <qfileinfo.h>
//-------------------------------------------------------------------
#include "scheditline.h"
#include "scheditdocbox.h"
#include "scheditpagedef.h"
//-------------------------------------------------------------------
#define TitleTextPageSizeIndex 4
//-------------------------------------------------------------------
class ScheditPageDef;
class ScheditTitleBlock;
class ColourValuePref;
//-------------------------------------------------------------------
/**
	@author James Fuller,,, <james@slakware12>
*/
class ScheditHeader : public QCanvasItem {
private:
	typedef struct _pagedef {
		QSize	page;
		int		hDiv;
		char	vDiv;
		
		QSize	getSize ( void );
		QRect	getRect ( void );
		int		getHdiv ( void );
		int		getVdiv ( void );
	} PageDef;

#pragma pack (push,1)
	typedef struct {
		char    ucType;
		char    cPageEnum;
		char    aucQue2 [2];
		char    ascilTag [12];       // Must be 'Schematic  ' !
	} hdr;
#pragma pack (pop)
	
	hdr 		header;
	QPointArray paHor;
	QPointArray paVer;
	QPointArray paHorText;
	QPointArray paVerText;
	QRect 		rectInner;
	
	QRect 	  rectPage;
	ScheditPageDef * pSpd;
	
	static ScheditLinePrefs * lpBackground;
	static ColourValuePref  * lpLine;
	static ScheditFontPrefs * lpText;
	
	QSize setupPage ( int iIndex );
	ScheditTitleBlock * pTitleBlock;
	
	static LabelIni  * laVisible;
	static QCheckBox * cbShowBorders;
		
public:
	ScheditHeader( QCanvas * canvas,
				   unsigned char * & puc );

    ~ScheditHeader();

	bool collidesWith ( const QCanvasItem * other ) const;

	bool collidesWith( const QCanvasSprite* cs,
						const QCanvasPolygonalItem* pi,
		   				const QCanvasRectangle* cr,
						const QCanvasEllipse* ce,
						const QCanvasText* ct) const;
	QRect boundingRect ( void ) const;
	void draw ( QPainter& painter );
	virtual int rtti () const;
	
	QRect getRect (bool& bDraw);
	QSize getSize ( void );
	QColor BackgroundColour;
	QSize setPageFromEnum   ( ScheditPageDef::ePageSize ps );
	void setFileData ( QFileInfo qfi );
	void setPage (void);

	static QSize getSize ( ScheditPageDef::ePageSize ps );
	
	static void setup ( KDialogBase * dlg );
	
	static void readProperties  ( KConfig * config );
		
	static void writeProperties ( KConfig * config );	
};

#endif
