/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include <kdialogbase.h>
#include <kconfig.h>
//---------------------------------------------------------------------------
#include <qcanvas.h>
#include <qpainter.h>
#include <qlayout.h>
//---------------------------------------------------------------------------
#include "scheditjunction.h"
#include "colourvaluepref.h"
#include "scheditline.h"
//---------------------------------------------------------------------------
//int ScheditJunction::iRad = 3;			// get diameter from database
//QBrush ScheditJunction::brush (Qt::blue);	// get colour from database

ColourValuePref * ScheditJunction::lp; 
//---------------------------------------------------------------------------
ScheditJunction::ScheditJunction(QCanvas * canvas, unsigned char * & puc)
				: QCanvasEllipse ( canvas ) {
	int iLength = sizeof ( JunctionHdr );
	memcpy ( &header, puc, iLength );
	puc += iLength;
	
	setX ( header.usStartX );
	setY ( header.usStartY );
	setSize ( 3, 3 );
	setBrush ( QBrush ( Qt::blue ) );
	show ();
}
//---------------------------------------------------------------------------
ScheditJunction::ScheditJunction ( QCanvas * canvas, QPoint qp )
	: QCanvasEllipse ( canvas ) {
	header.usStartX = qp.x ();
	header.usStartY = qp.y ();		
	setX ( header.usStartX );
	setY ( header.usStartY );
	setSize ( 3, 3 );
	setBrush ( QBrush ( Qt::blue ) );
	show ();
}
//---------------------------------------------------------------------------
ScheditJunction::~ScheditJunction(){
}
//---------------------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int ScheditJunction::rtti () const {
	return 1006;
}
//---------------------------------------------------------------------------
void ScheditJunction::draw ( QPainter& painter ) {
	int iRadius;
	
	lp->setupPainter ( painter );
	iRadius = lp->getValue ();
	setSize ( iRadius + 1, iRadius + 1);
	QBrush brush = painter.brush ();
	brush.setStyle ( Qt::SolidPattern );
	painter.setBrush ( brush );
	painter.drawEllipse ( boundingRect () );
}
//---------------------------------------------------------------------------
// Properties
//---------------------------------------------------------------------------
void ScheditJunction::readProperties ( KConfig * config) {	
	lp->readGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditJunction::writeProperties ( KConfig * config ) {	
	lp->writeGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditJunction::setup ( KDialogBase * dlg ) {	
	QString qsRoot = "Junction"; 
	
	lp = new ColourValuePref (dlg, qsRoot, qsRoot);
	lp->setSpinBoxTitle ( "Radius" );
	lp->adjustForCheckBoxes ();	
	
// Create objects here for preference screen page
// We are making a grid of 9 junctions on three line styles
	QPoint		qpStart;
	QPoint		qpEnd;
	QPoint		qp;
	QCanvas * 	canvas = lp->getCanvas ();
	int			xStyle;
	int			yStyle;
	QCanvasItem * si;
	
// Vertical lines
	for ( yStyle = 1; yStyle <=3; yStyle++ ) {
		qp.setX ( point ( yStyle ) );
		qpStart = QPoint ( qp.x (), 10 );
		qpEnd   = QPoint ( qp.x (), 130 );
		si = new ScheditLine ( canvas, qpStart, qpEnd, yStyle );
		si->setZ ( 1 );
	} /* endfor */
	
// Horizontal Lines
	for ( xStyle = 1; xStyle <=3; xStyle++ ) {
		qp.setY ( point ( xStyle ) );
		qpStart = QPoint ( 10, qp.y () );
		qpEnd   = QPoint ( 130, qp.y () );
		si = new ScheditLine ( canvas, qpStart, qpEnd, xStyle );
		si->setZ ( 2 );
	} /* endfor */
	
// Junctions
	for ( int xStyle = 1; xStyle <=3; xStyle++ )
		for ( int yStyle = 1; yStyle <=3; yStyle++ ) {
			qp = QPoint ( point ( xStyle ), 
						  point ( yStyle ) );
			si = new ScheditJunction ( canvas, qp ); 
			si->setZ ( 3 );
		} /* endfor */
}
//---------------------------------------------------------------------------
int ScheditJunction::point ( int iStyle ) {	
	int iStart = -10;
	int iGap   = 40;
	
	return iStart + ( iStyle * iGap );
}
//---------------------------------------------------------------------------


