/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include <qpainter.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpixmap.h>
//---------------------------------------------------------------------------
#include <kdialogbase.h>
#include <kconfig.h>
#include <klocale.h>
//---------------------------------------------------------------------------
#include "scheditline.h"
#include "spinvalue.h"
#include "scheditprefs.h"
//---------------------------------------------------------------------------
ScheditLine::ScheditLine(QCanvas * canvas, 
						 unsigned char * & puc )
			: QCanvasLine ( canvas ) {
	
	int iLength = sizeof ( WireHdr );
	
	memcpy ( &header, puc, iLength );
	puc += iLength;
	common ();
}
//---------------------------------------------------------------------------
ScheditLine::ScheditLine ( 	QCanvas *	canvas, 
							QPoint		qpStart,
							QPoint		qpEnd,
							int			iStyle ) 
			: QCanvasLine ( canvas ) {
	header.ucType   = 0x10;
	header.usStartX = qpStart.x ();
	header.usStartY = qpStart.y ();
	header.usEndX   = qpEnd.x ();
	header.usEndY   = qpEnd.y ();
	header.ucStyle = (unsigned char) iStyle;
	common ();
}
//---------------------------------------------------------------------------
void ScheditLine::common ( void ) {
	setPoints ( header.usStartX, header.usStartY, 
				header.usEndX,   header.usEndY );
	show ();
}
//---------------------------------------------------------------------------
//ScheditLine::~ScheditLine () {
//	for ( ScheditLinePrefs * lp = aSetupPages.first ();
//		  lp;
//		  lp = aSetupPages.next () ) {
//		delete lp; 
//	} /* endfor */
//}
//---------------------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int ScheditLine::rtti () const {
	return 1005;
}
//---------------------------------------------------------------------------
void ScheditLine::draw ( QPainter& painter ) {
	ScheditLinePrefs * lp;
	lp = aSetupPages.at (header.ucStyle - 1);
	if ( lp ) painter.setPen ( lp->getPen () ); 
	painter.drawLine ( startPoint (), endPoint () );
}
//---------------------------------------------------------------------------
void ScheditLine::moveBy ( double x, double y ) {
	QPoint qpMoveBy ( x, y );
	
	QPoint qpStart = startPoint () + qpMoveBy;
	QPoint qpEnd   = endPoint   () + qpMoveBy;
	setPoints ( qpStart.x (), qpStart.y (),
				qpEnd.x (),   qpEnd.y () ); 
}	
//---------------------------------------------------------------------------
void ScheditLine::readProperties ( KConfig * config) {	
	ScheditLinePrefs * lp;

	for ( lp = aSetupPages.first (); lp; lp = aSetupPages.next () )
		lp->readGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditLine::writeProperties ( KConfig * config ) {	
	ScheditLinePrefs * lp;

	for ( lp = aSetupPages.first (); lp; lp = aSetupPages.next () )
		lp->writeGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditLine::setup ( KDialogBase * dlg ) {	
	ScheditLinePrefs * lp;
	QString qsTitle;
	QString qsKey;
	typedef QValueList<QString> SizeList;
	SizeList list;
	QString qs;
	
	list << "Thin Signal" 
		 << "Thick Signal"
		 << "Bus"
		 << "Dashed";
	
	SizeList::iterator it;
	for ( it = list.begin (); it != list.end (); ++it ) {
		qs = *it;
		qsTitle.sprintf ( "Line style '%s'", qs.ascii () );
		qsKey.sprintf ( "Line.%s", qs.ascii () );
		lp = new ScheditLinePrefs (dlg,
								   qsTitle,
								   qsKey );
		aSetupPages.append ( lp );
	} /* endfor */
}
//---------------------------------------------------------------------------
// Prefs class for line types
// Colour, thickness, style
//---------------------------------------------------------------------------
QPtrList<ScheditLinePrefs> ScheditLine::aSetupPages;
//---------------------------------------------------------------------------
ScheditLinePrefs::ScheditLinePrefs ( KDialogBase * dlg,
									 QString       qsTitle, 
		  							 QString       qsKey )
	: ScheditPrefs (dlg, qsTitle, qsKey)
	, si1 ( this )
	, si2 ( this )
	, laColour  ( "Colour", this )
	, dcb       ( this, "Colour", &laColour )
	, laStyle   ( "Style", this )
	, ls        ( this, "Style", &laStyle )
	, laWidth   ( "Width", this )
	, svWidth   ( 1, 5, &laWidth, 1, this, "Width" ) {
// Bold section titles
	embolden ( &laColour );
	embolden ( &laStyle );
	embolden ( &laWidth );

// Connect the Colour and Width signals to the 'render' function
	connect ( &svWidth, SIGNAL (valueChanged(int) ), 
			  this, SLOT ( render () ) );

	connect ( &dcb, SIGNAL(colorSelected(int)),
			   this, SLOT(render()));
	
	connect ( &ls, SIGNAL(selectionChanged(int)),
			   this, SLOT(render()));
	
	adjustSize ();
	svWidth.setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
	ls.setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Fixed);
}
//---------------------------------------------------------------------------
void ScheditLinePrefs::hideValue (void ) {
	laWidth.hide ();
	svWidth.hide ();
}
//---------------------------------------------------------------------------
void ScheditLinePrefs::hideStyle (void ) {
	laStyle.hide ();
	ls.hide ();
}
//---------------------------------------------------------------------------
void ScheditLinePrefs::renderPage () {
	data.pen = getPen ();
	ls.setPen ( data.pen );
}
//---------------------------------------------------------------------------
void ScheditLinePrefs::readPreferences ( KConfigGroup * config ) {
	int iValue;
											 
// Colour
	dcb.read ( config, sLastKey );
	
// Width
//	if ( svWidth.isVisible () ) 
		svWidth.read ( config, sLastKey );
	
// Style
//	if ( ls.isVisible () ) {
		iValue = laStyle.read ( config, sLastKey );
		ls.setSelected ( iValue );
//	} /* endif */
}
//---------------------------------------------------------------------------
void ScheditLinePrefs::writePreferences ( KConfigGroup * config ) {
	dcb.write ( config, sLastKey );
	
//	if ( ls.isVisible () )
		ls.write ( config, sLastKey );
		
//	if ( svWidth.isVisible () ) 
		svWidth.write ( config, sLastKey );
}
//---------------------------------------------------------------------------
QPen ScheditLinePrefs::getPen (void) {
	QPen pen = ls.getPen ();
	pen.setWidth ( svWidth.value () );
	pen.setColor ( dcb.getColour () );
	return pen;	
}
//---------------------------------------------------------------------------
void ScheditLinePrefs::setupPainter (QPainter& painter) {
	ls.setupPainter (painter);
}
//---------------------------------------------------------------------------
#include "scheditline.moc"



