/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SCHEDITLINE_H
#define SCHEDITLINE_H
//---------------------------------------------------------------------------
#include <qcanvas.h>
#include <qptrlist.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qpixmap.h>
#include <qpainter.h>
//---------------------------------------------------------------------------
#include "spinvalue.h"
#include "doscolourbox.h"
#include "linestyle.h"
#include "scheditprefs.h"
//---------------------------------------------------------------------------
class KDialogBase;
class SpinValue;
//-------------------------------------------------------------------
// class ScheditLinePrefs - being a dialog window 
//-------------------------------------------------------------------
/**
	@author James Fuller,,, <james@slakware12>
 */
class ScheditLinePrefs : public ScheditPrefs {
	Q_OBJECT
public:
	typedef struct {
		QPen pen;
	} Data;
	
private:
	LabelIni 	 laColour;
	DosColourBox dcb;
	QLabel		 si1;
	LabelIni 	 laStyle;
	LineStyle 	 ls;
	QLabel		 si2;
	LabelIni 	 laWidth;
	SpinValue 	 svWidth;	
	Data 		 data;
	
	void readPreferences  ( KConfigGroup * config );
	void writePreferences ( KConfigGroup * config );	
			
protected:
	void renderPage ();

public:
	ScheditLinePrefs (KDialogBase * dlg,
					  QString       qsTitle, 
	   				  QString       qsKey );
	void hideValue (void);
	void hideStyle (void);
	QPen getPen (void);
	void setupPainter (QPainter& painter);
};
//-------------------------------------------------------------------
/**
	@author James Fuller,,, <james@slakware12>
*/
class ScheditLine : public QCanvasLine {
private:
#pragma pack (push,1)
	typedef struct {
		unsigned char 	   ucType;
		unsigned short int usStartX;
		unsigned short int usStartY;
		unsigned short int usEndX;
		unsigned short int usEndY;
		unsigned char 	   ucStyle;
		unsigned short int ausScrap [3];
	} WireHdr;
#pragma pack (pop)

	WireHdr header;
	
	void common ( void );
	
	static QPtrList<ScheditLinePrefs> aSetupPages;
	
//	static ScheditLinePrefs::Data data;

public:
	ScheditLine ( QCanvas * canvas, 
				  unsigned char * & puc );

	ScheditLine ( QCanvas *	canvas, 
					QPoint	qpStart,
					QPoint	qpEnd,
					int		iStyle );

//    ~ScheditLine();
	
	int rtti () const;
	
	void draw ( QPainter& painter );
	
	void moveBy ( double x, double y );
	
	static void setup ( KDialogBase * dlg );
	
	static void readProperties ( KConfig * config );
	
	static void writeProperties ( KConfig * config );	
};

#endif
