/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include <kdialogbase.h>
#include <kconfig.h>
//---------------------------------------------------------------------------
#include <qcheckbox.h>
//---------------------------------------------------------------------------
#include "scheditprefs.h"
//---------------------------------------------------------------------------
// ScheditPrefs
//---------------------------------------------------------------------------
ScheditPrefs::prefsList ScheditPrefs::list;
//---------------------------------------------------------------------------
ScheditPrefs::ScheditPrefs (KDialogBase * dlg,
							QString qsTitle,
	   						QString qsKey )
	: sKey (qsKey)
	, layout (this) 
	, QFrame(addPage (dlg, qsTitle, qsKey )) {
	layout.setAutoAdd(true);
	list.append ( this );
	pLocalData = NULL;
	pObjectData = NULL;
	iSize = 0;
	setName ( qsKey );
	
//	QStringList sl = QStringList::split ( ".", sKey );
	sLastKey = "";//sl.last ();
	
// ensure page updates before being shown
	connect ( dlg,  SIGNAL(aboutToShowPage(QWidget*)),
			  this, SLOT(render()));
}
//---------------------------------------------------------------------------
void ScheditPrefs::setPage ( int iIndex ) {
	iPage = iIndex;
}
//---------------------------------------------------------------------------
void ScheditPrefs::render ( int iIndex ) {
	for ( ScheditPrefs * pObj = list.first (); pObj; pObj = list.next () ) {
		if ( iIndex != pObj->iPage ) continue;
		pObj->render ();
		break;
	} /* endfor */
}
//---------------------------------------------------------------------------
QFrame * ScheditPrefs::addPage ( KDialogBase * dlg,
								 QString 	   qsTitle,
								 QString 	   qsKey) {
	QFrame * frame;
	QPoint 	 qp ( 0, 0 );
	QStringList sl;
	QRect   rect;

// Spaces ad an extra element the string list decomposition
// replace with a nominal value and remove at end
	qsKey.replace ( ' ', '@' );
	sl = QStringList::split ( ".", qsKey );
	for ( QStringList::Iterator it = sl.begin(); it != sl.end(); ++it )
		(*it).replace ( '@', ' ' );

	frame = dlg->addPage ( sl, qsTitle );
	rect = frame->geometry ();
	rect.setWidth ( 400 );
	frame->setGeometry ( rect );
	iPage = dlg->pageIndex ( frame );
	return frame;
}
//---------------------------------------------------------------------------
void ScheditPrefs::renderPage () {	// stub
}
//---------------------------------------------------------------------------
void ScheditPrefs::render () {
	renderPage ();					// virtual
}
//---------------------------------------------------------------------------
void ScheditPrefs::embolden ( QWidget * w ) {
// Bold section titles
	QFont font = w->font ();
	font.setWeight (QFont::DemiBold );
	w->setFont ( font );
}
//---------------------------------------------------------------------------
// Communications to actual object data this prefs object refers to
//---------------------------------------------------------------------------
// This is called when the dialog is viewed
void ScheditPrefs::load () {
	ScheditPrefs * lp;
	
	for ( lp = list.first (); lp; lp = list.next () )
		copy ( lp->pLocalData, lp->pObjectData, lp->iSize );
}
//---------------------------------------------------------------------------
// This is called when either 'Apply or Ok' are clicked
void ScheditPrefs::save () {
	ScheditPrefs * lp;

	for ( lp = list.first (); lp; lp = list.next () )
		copy ( lp->pObjectData, lp->pLocalData, lp->iSize );
}
//---------------------------------------------------------------------------
// Safe copy
void ScheditPrefs::copy ( void * pDst, void * pSrc, int iCount ) {
	if ( pDst == NULL ) return;
	if ( pSrc == NULL ) return;
	memcpy ( pDst, pSrc, iCount );
}
//---------------------------------------------------------------------------
void ScheditPrefs::readGroup ( KConfig * config ) {
	bool		bFlag;
	QCheckBox * cb;		
	KConfigGroup group ( (KConfigBase *) config, sKey );
	readPreferences ( &group);
	
	for ( cb = apcb.first (); cb; cb = apcb.next () ) {
		bFlag = group.readBoolEntry ( cb->name () );
		cb->setChecked ( bFlag );
	} /* endfor */
}
//---------------------------------------------------------------------------
void ScheditPrefs::writeGroup ( KConfig * config ) {		
	bool		bFlag;
	QCheckBox * cb;		
	KConfigGroup group ( (KConfigBase *) config, sKey );
	writePreferences ( &group);
	
	for ( cb = apcb.first (); cb; cb = apcb.next () ) {
		bFlag = cb->isChecked ();
		group.writeEntry ( cb->name (), bFlag );
	} /* endfor */
}
//---------------------------------------------------------------------------
QCheckBox * ScheditPrefs::makeCb ( QString qs ) {		
	QCheckBox * cb = new QCheckBox ( qs, this );
	qs.remove ( ' ' );		// remove readability spaces
	qs.remove ( '&' );		// remove hot key information
	cb->setName ( qs );
	apcb.append ( cb );		// add to list of checkboxes to load/save
	connect ( cb,   SIGNAL ( clicked () ),
			  this, SLOT   ( render  () ) );
	return cb;
}
//---------------------------------------------------------------------------
#include "scheditprefs.moc"

