/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//---------------------------------------------------------------------------
#include <kconfig.h>
//---------------------------------------------------------------------------
#include <qfileinfo.h>
#include <qcheckbox.h>
//---------------------------------------------------------------------------
#include "schedittitleblock.h"
#include "pointhelper.h"
#include "colourvaluepref.h"
//---------------------------------------------------------------------------
extern ScheditLocatedText sltFile;
extern ScheditLocatedText sltDate;
extern ScheditLocatedText sltSize2;
//---------------------------------------------------------------------------
ScheditFontPrefs * ScheditTitleBlock::lpText;
ColourValuePref  * ScheditTitleBlock::lpLine;

QCheckBox * ScheditTitleBlock::cbBlock   = NULL;
LabelIni  * ScheditTitleBlock::laVisible = NULL;

QLabel * ScheditTitleBlock::pixLabel = NULL;
//---------------------------------------------------------------------------
ScheditTitleBlock::ScheditTitleBlock ( QCanvas * canvas ) 
				: QCanvasRectangle ( canvas ) 
				, sltSize2 	( 12,	56, ""			)
				, sltDate	( 7,	69, "Date:"		)
				, sltFile	( 7, 	80, "File:"	 	) {
	QPointArray pa;
	QRect qr;
	
	sltSize2.move ( aslt );
	sltDate.move  ( aslt );
	sltFile.move  ( aslt );
	
// Vertical Boundary
	pa.resize ( 0 );
	pa  << QPoint ( 0, 0 )
		<< QPoint ( 0, 80 );
	paTitleBlock += pa;
	
// Horizontal Boundary
	pa.resize ( 0 );
	pa  << QPoint ( 0, 0 )
			<< QPoint ( 350, 0 );
	paTitleBlock += pa;
	
// Horizontal Inner Lines
	pa.translate ( 0, 29 );
	paTitleBlock += pa;
	pa.translate ( 0, 30 );
	paTitleBlock += pa;
	pa.translate ( 0, 11 );
	paTitleBlock += pa;
	
// Vertical Inner Lines
	pa.resize ( 0 );
	pa  << QPoint ( 50, 30 )
			<< QPoint ( 50, 59 );
	paTitleBlock += pa;
	pa.translate ( 200, 0 );
	paTitleBlock += pa;
	
	pa.resize ( 0 );
	pa  << QPoint ( 200, 60 )
			<< QPoint ( 200, 80 );
	paTitleBlock += pa;
	
// Save TitleBlock Size
	qr = paTitleBlock.boundingRect ();
	setSize ( qr.width (), qr.height () );
	
	qpBlock = qr.bottomRight ();	
	
	show ();
}
//---------------------------------------------------------------------------
ScheditTitleBlock::~ScheditTitleBlock()
{
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::setOffset ( QPoint qp ) {
	qpOffset = qp - qpBlock;
	move ( qpOffset.x (),
			qpOffset.y () );
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::setFileData ( QFileInfo qfi ) {
	
	QString   qs;
	QDateTime dt;
	
	// Information
	qs = qfi.fileName ();
	sltFile.append ( qs );
	
	dt = qfi.lastModified ();
	qs = dt.toString ( "d-MMM  yyyy" );
	sltDate.append ( qs );
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::setPageSize ( QString qs ) {	
	sltSize2.append ( qs );
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::draw ( QPainter& painter ) {
	bool bDraw = cbBlock->isChecked ();
	if ( ! bDraw ) return;
	painter.save ();
	painter.translate ( x (), y () );
	
	lpLine->setupPainter ( painter );
	painter.drawLineSegments ( paTitleBlock );
	
	lpText->setupPainter 		( painter );
	ScheditLocatedText::drawAll ( painter );
	ScheditLocatedText::drawAll ( painter, aslt );
	
	painter.restore ();
}
//---------------------------------------------------------------------------
// Preferences
//---------------------------------------------------------------------------
void ScheditTitleBlock::setup ( KDialogBase * dlg ) {	
	QString qsRoot = "Title Block";
										 
	lpLine = new ColourValuePref (dlg, 
								   qsRoot + QString (" Line"),
								   "Title.Line" ); 
	
	new QLabel ( lpLine );
	laVisible = new LabelIni  ( "Visibility",       lpLine );
	cbBlock   = lpLine->makeCb ( "Show Title Block" );
	lpLine->embolden ( laVisible );
	lpLine->adjustForCheckBoxes ();	
	
	new QLabel ( "Note - XRef (borders) must be displayed to show Title Block", lpLine );

	lpText = new ScheditFontPrefs (dlg, 
								   qsRoot + QString (" Text"),
								   QString ("Title.Text") ); 
	
// Create sample Title Block	
	QCanvas * canvas = lpLine->getCanvas ();
	ScheditTitleBlock * pObj = new ScheditTitleBlock ( canvas );
	pObj->move ( 10, 10 );
	pObj->sample ();
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::readProperties ( KConfig * config ) {	
	lpLine->readGroup ( config );
	lpText->readGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::writeProperties ( KConfig * config ) {	
	lpLine->writeGroup ( config );
	lpText->writeGroup ( config );
}
//---------------------------------------------------------------------------
void ScheditTitleBlock::sample ( void ) {
	sltFile.append ( "SampleFile.s01" );
	sltDate.append ( "10-Feb  2014" );
	sltSize2.append ( "A4" );
}	
//---------------------------------------------------------------------------
void ScheditTitleBlock::drawShape ( QPainter & painter ) {	
	draw ( painter );
}
//---------------------------------------------------------------------------



