/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//-------------------------------------------------------------------
#include "schematux.h"
#include "pref.h"
#include "qstatusbarkeyval.h"
//-------------------------------------------------------------------
#include <qdragobject.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qradiobutton.h>
//-------------------------------------------------------------------
#include <kprinter.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kstatusbar.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>
#include <qmessagebox.h>
//-------------------------------------------------------------------
#include <scheditfile.h>
//-------------------------------------------------------------------
Schematux::Schematux ()
    : KMainWindow ( 0, "Schematux" )
    ,  m_view(new SchematuxView ( this, &canvas ) )
    ,  m_printer ( 0 ) {
    // accept dnd
    setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    QStatusBar * sb = statusBar();
	kvMsg   = new QStatusBarKeyVal ( "Message", "[                ]", this );
	kvMouse = new QStatusBarKeyVal ( "Mouse", "8888,8888", this );
	kvObjects = new QStatusBarKeyVal ( "Objects", "[                   ]", this );
	sb->addWidget ( kvMsg );
	sb->addWidget ( kvMouse );
	sb->addWidget ( kvObjects );
	sb->show();

	
    // Apply the create the main window and ask the mainwindow to
	// automatically save settings if changed: window size, toolbar
    // position, icon size, etc.  Also to add actions for the statusbar
	// toolbar, and keybindings if necessary.
    setupGUI();

	canvas.resize ( 500, 500 );
	
	sf = new ScheditFile ( &canvas );
	KConfig * config = KGlobal::config();
	readProperties ( config );
	dlgPrefs = new SchematuxPreferences ();
	
	sf->setup ( dlgPrefs );	
	dlgPrefs->setModal (true);
	dlgPrefs->resize ( 749, 651 );
	dlgPrefs->unfoldTreeList ();

	connect (dlgPrefs, SIGNAL(signalApply()),
			 m_view, SLOT(slotRedraw(void)));
	
	connect (dlgPrefs, SIGNAL(signalSave()),
			 this, SLOT(fileSave()));
	
	connect (m_view, SIGNAL(mouseMove(const QString&)),
			 this, SLOT(mouseMove(const QString&)));
	
	connect (m_view, SIGNAL(objList(const QString&)),
			 this, SLOT(objList(const QString&)));
	
	sf->readProperties ( config );	
	QRect rectCanvas = canvas.rect ();	
	QRect rectView   = m_view->contentsRect ();
	
	// put up a window to accept different zoom settings
	ZoomWin dlg ( rectCanvas, rectView );
	updateZoomDlg (dlg);
	applyZoom ( dlg );	
	
	QString url = config->readPathEntry("lastURL");
	if (!url.isEmpty() ) {
		openURL(KURL(url));
		qsStartDir = KURL (url).directory ();
	} /* endif */
}
//-------------------------------------------------------------------
Schematux::~Schematux () {
	delete sf;
	delete dlgPrefs;
}
//-------------------------------------------------------------------
void Schematux::load ( const KURL& url ) {
    QString target;
    // the below code is what you should normally do.  in this
    // example case, we want the url to our own.  you probably
    // want to use this code instead for your app

    #if 0
    // download the contents
    if (KIO::NetAccess::download(url, target))
    {
        // set our caption
        setCaption(url);

        // load in the file (target is always local)
        loadFile(target);

        // and remove the temp file
        KIO::NetAccess::removeTempFile(target);
    }
    #endif
	
//    openURL(url);
}
//-------------------------------------------------------------------
void Schematux::setupActions () {
    KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
    KStdAction::open(this, SLOT(fileOpen()), actionCollection());
    KStdAction::save(this, SLOT(fileSave()), actionCollection());
    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
    KStdAction::print(this, SLOT(filePrint()), actionCollection());
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());
	KStdAction::zoom ( this, SLOT(viewZoom()),actionCollection ());
	fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const 
			KURL&)), actionCollection());

    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    // this doesn't do anything useful.  it's just here to illustrate
    // how to insert a custom menu and menu item
//    KAction *custom = new KAction ( i18n ( "Cus&tom Menuitem" ), 
//									0,
//                                    this, 
//									SLOT ( optionsPreferences () ),
//                                  	actionCollection(), 
//									"custom_action" );
}
//-------------------------------------------------------------------
void Schematux::saveProperties ( KConfig * config ) {
	QString url = currentURL ();
	QString qs;
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

    if (!currentURL().isEmpty()) {
#if KDE_IS_VERSION(3,1,3)
        config->writePathEntry ( "lastURL", url );
#else
        config->writeEntry("lastURL", url );
#endif
	}
	qsStartDir = KURL (url).directory ();
	
// Zoom data
	KConfigGroup zoomGroup( config, "Zoom" );
	qs.sprintf ( "%d", (int) zp );
	zoomGroup.writeEntry ( "Pref", qs );
	qs.sprintf ( "%d", (int) iZoomPercent );
	zoomGroup.writeEntry ( "Value", qs );
	
// Component data
	sf->saveProperties ( config );	
}
//-------------------------------------------------------------------
void Schematux::readProperties ( KConfig * config ) {
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'
	QString qs;

// Zoom
	KConfigGroup zoomGroup( config, "Zoom" );
	zp = ZoomWin::none;
	iZoomPercent = 100;
	
	qs = zoomGroup.readEntry ( "Pref", "" );
	if ( qs.isEmpty () )
		zp = ZoomWin::none;
			
	else
		zp = (ZoomWin::ZoomPref) qs.toInt ();
	
	qs = zoomGroup.readEntry ( "Value", "" );
	if ( qs.isEmpty () )
		iZoomPercent = 100;
			
	else
		iZoomPercent =  qs.toInt ();
	}
//-------------------------------------------------------------------
void Schematux::dragEnterEvent ( QDragEnterEvent * event ) {
    // accept uri drops only
    event->accept(KURLDrag::canDecode(event));
}
//-------------------------------------------------------------------
void Schematux::dropEvent ( QDropEvent *event ) {
    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    KURL::List urls;

    // see if we can decode a URI.. if not, just ignore it
    if (KURLDrag::decode(event, urls) && !urls.isEmpty())
    {
        // okay, we have a URI.. process it
        const KURL &url = urls.first();

        // load in the file
        load(url);
    }
}
//-------------------------------------------------------------------
void Schematux::fileNew () {
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // create a new window
    (new Schematux)->show();
}
//-------------------------------------------------------------------
void Schematux::fileOpen () {
	QString qsFilter = "*.s01|Schedit Files";
	
    KURL url = KFileDialog::getOpenURL ( qsStartDir, 			// start directory
										 qsFilter, 				// Filter	
		   								 this, 
	 									 i18n("Open Location"));
	fileOpen ( url );
}
//-------------------------------------------------------------------
void Schematux::slotFileOpenRecent(const KURL& url) {
	//slotStatusMsg(i18n("Opening file..."));
	fileOpen ( url );
}
//-------------------------------------------------------------------
void Schematux::fileOpen (const KURL& url) {
	QString qs;
	KConfig * config;
	
	if (url.isEmpty()) return;			// Nowhere from
	config = KGlobal::config();
	
//which of these is better ?	
//	qsStartDir = url.directory ();
	qsStartDir = KURL (url).directory ();
	
// finally - open the file !
	openURL(url);
	
#if KDE_IS_VERSION(3,1,3)
	config->writePathEntry ( "lastURL", currentURL () );
#else
	config->writeEntry("lastURL", currentURL () );
#endif
}
//-------------------------------------------------------------------
void Schematux::fileSave()
{
    // this slot is called whenever the File->Save menu is selected,
    // the Save shortcut is pressed (usually CTRL+S) or the Save toolbar
    // button is clicked

    // save the current file
	//KConfig config("myapprc");
	KConfig * config = KGlobal::config();
	
	//QMessageBox ( NULL, "Hi there", config->
	
	saveProperties ( config );
}
//-------------------------------------------------------------------
void Schematux::fileSaveAs () {
    // this slot is called whenever the File->Save As menu is selected,
    KURL file_url = KFileDialog::getSaveURL();
    if (!file_url.isEmpty() && file_url.isValid())
    {
        // save your info, here
    }
}
//-------------------------------------------------------------------
void Schematux::filePrint () {
    // this slot is called whenever the File->Print menu is selected,
    // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
    // button is clicked
    if (!m_printer) m_printer = new KPrinter;
    if (m_printer->setup(this))
    {
        // setup the printer.  with Qt, you always "print" to a
        // QPainter.. whether the output medium is a pixmap, a screen,
        // or paper
        QPainter p;
        p.begin(m_printer);

        // we let our view do the actual printing
        QPaintDeviceMetrics metrics(m_printer);

        m_view->print(&p, metrics.height(), metrics.width());

        // and send the result to the printer
        p.end();
    }
}
//-------------------------------------------------------------------
void Schematux::optionsPreferences () {
    // popup some sort of preference dialog, here
    //SchematuxPreferences dlg;
	
	// can we save the last viewed preferences page ?
	// yes - use 'activePageIndex ()
	// restore with 'showPage ()'
	
    dlgPrefs->show();
}
//-------------------------------------------------------------------
void Schematux::viewZoom () {
	QRect rectCanvas = canvas.rect ();	
	QRect rectView   = m_view->contentsRect ();
	
	// put up a window to accept different zoom settings
	ZoomWin dlg ( rectCanvas, rectView );
	updateZoomDlg (dlg);		// show current settings
	if ( dlg.exec () ) {
		applyZoom ( dlg );
	}
}
//-------------------------------------------------------------------
// Status Bars
//-------------------------------------------------------------------
void Schematux::changeStatusbar ( const QString& text ) {
    // display the text on the statusbar
    kvMsg->setVal (text);
}
//-------------------------------------------------------------------
void Schematux::mouseMove ( const QString& text ) {
	kvMouse->setVal ( text );
}
//-------------------------------------------------------------------
void Schematux::objList ( const QString& text ) {
	kvObjects->setVal ( text );
}
//-------------------------------------------------------------------
// Caption
//-------------------------------------------------------------------
void Schematux::changeCaption ( const QString& text ) {
    // display the text on the caption
    setCaption(text);
}
//-------------------------------------------------------------------
// Zoom functions
//-------------------------------------------------------------------
// retrieves zoom preferences from dialog window
// and implements them
void Schematux::applyZoom ( ZoomWin& dlg ) {
// gets the zoom percentage from the text edit window
	iZoomPercent = dlg.zoomPercent ();
	zp = dlg.zoomStyle ();
	double  dZoom = (double) iZoomPercent / 100.0;
	QWMatrix wm ( dZoom, 0, 0, dZoom, 0, 0 );
	m_view->setWorldMatrix ( wm );
	m_view->repaintContents ();
}
//-------------------------------------------------------------------
// Places zoom preferences in dialog window
void Schematux::updateZoomDlg ( ZoomWin& dlg ) {
	dlg.setZoomPercent ( iZoomPercent );
	switch ( zp ) {
	case ZoomWin::page:
		dlg.rbPage->setChecked ( true );
		break;
		
	case ZoomWin::user:
		dlg.rbUser->setChecked ( true );
		break;
		
	case ZoomWin::width:
		dlg.rbWidth->setChecked ( true );
		break;
		
	case ZoomWin::height:
		dlg.rbHeight->setChecked ( true );
		break;
		
	case ZoomWin::none:
	default:
		dlg.slotNone ();
		dlg.rbNone->setChecked ( true );
		break;	
	} /* endswitch */	
}
//-------------------------------------------------------------------
// File Handling stuff
//-------------------------------------------------------------------
QString Schematux::currentURL () {
	return qsLastFile;
}
//-------------------------------------------------------------------
void Schematux::openURL(QString url) {
	openURL ( KURL ( url ) );
}
//-------------------------------------------------------------------
void Schematux::openURL ( const KURL& url ) {
	QString localfilename;
	QString s;

	if (url.isLocalFile()) {
		localfilename = url.path();
		qsLastFile = localfilename;

	} else 
		if (!KIO::NetAccess::download ( url, localfilename) ) {
			qsLastFile = url.fileName ();
			changeStatusbar(QString( "fail" ) );
			return;
		}// do something !

	changeStatusbar("Loaded");

	// open file 's'
	sf->loadFromFile ( localfilename );
	update ();
	m_view->repaintContents ();
	
// change caption
	QString qsTitle;
	
	qsTitle = url.url ();
	changeCaption ( qsTitle );	
}
//-------------------------------------------------------------------
#include "schematux.moc"
//-------------------------------------------------------------------

