/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _SCHEMATUX_H_
#define _SCHEMATUX_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kmainwindow.h>

#include "schematuxview.h"
#include "zoomwin.h"
#include <qcanvas.h>

class KPrinter;
class KURL;
class SchematuxPreferences;
class KRecentFilesAction;
class QStatusBarKeyVal;

/**
 * This class serves as the main window for Schematux.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author James Fuller,,, <james@slakware12>
 * @version 0.1
 */
class Schematux : public KMainWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    Schematux();

    /**
     * Default Destructor
     */
    virtual ~Schematux();

    /**
     * Use this method to load whatever file/URL you have
     */
    void load(const KURL& url);

protected:
    /**
     * Overridden virtuals for Qt drag 'n drop (XDND)
     */
    virtual void dragEnterEvent(QDragEnterEvent *event);
    virtual void dropEvent(QDropEvent *event);

protected:
    /**
     * This function is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveProperties(KConfig *);

    /**
     * This function is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref saveProperties
     */
    void readProperties(KConfig *);


private slots:
    void fileNew ();
	void fileOpen ();
	void fileSave ();
    void fileSaveAs ();
	void filePrint ();
	void viewZoom ();
	void optionsPreferences ();
	void slotFileOpenRecent(const KURL& url);

	void changeStatusbar(const QString& text);
    void changeCaption(const QString& text);
	void mouseMove ( const QString& text );
	void objList ( const QString& text );

    /**
	 * Random 'get' function
	 */
	QString currentURL();

    /**
	 * Random 'set' function accessed by DCOP
	 */
	virtual void openURL(QString url);

    /**
	 * Random 'set' function
	 */
	virtual void openURL(const KURL& url);
	
private:
	KRecentFilesAction*fileOpenRecent; 
	
	ScheditFile   * sf;	
	QString 		qsLastFile;
	
	int    iZoomPercent;
//	double dZoom;
	ZoomWin::ZoomPref zp;
	
	QString qsStartDir;
	
	QStatusBarKeyVal * kvMsg;
	QStatusBarKeyVal * kvMouse;	
	QStatusBarKeyVal * kvObjects;	
	
	void setupAccel();
    void setupActions();
	
	void applyZoom ( ZoomWin& dlg );
	void updateZoomDlg ( ZoomWin& dlg );
	void fileOpen (const KURL& url);

private:
	QCanvas canvas;

    SchematuxView * m_view;

    KPrinter   *m_printer;
	
	SchematuxPreferences * dlgPrefs;
};

#endif // _SCHEMATUX_H_

