/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-------------------------------------------------------------------
#include <kurl.h>
#include <ktrader.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kio/netaccess.h>
//-------------------------------------------------------------------
#include <qcanvas.h>
#include <qwmatrix.h>
#include <qcursor.h>
#include <qapplication.h>
#include <qpainter.h>
#include <qlayout.h>
//-------------------------------------------------------------------
#include "schematuxview.h"
#include "ScheditBitmap.h"
//-------------------------------------------------------------------
SchematuxView::SchematuxView ( QWidget * parent, 
								 QCanvas * pCanvas )
		      : QCanvasView    ( 		   parent )
			  , timMouse ( this, "mouseTimer" )
			  , DCOPObject ( "SchematuxIface" ) {
	QRect rect;
	QFont font ( "Monospace", 10 );

	enableScrollBars ( true );

    // setup our layout manager to automatically add our widgets
    QHBoxLayout *top_layout = new QHBoxLayout ( this );
    top_layout->setAutoAdd  ( true );
	setCanvas ( pCanvas );
    setDragAutoScroll ( true );

	connect ( this, SIGNAL (horizontalSliderReleased () ),
			  this, SLOT(slotRedraw () ) );
	
	connect ( this, SIGNAL (verticalSliderReleased () ),
			  this, SLOT(slotRedraw () ) );

	connect ( &timMouse, SIGNAL (timeout () ),
			  this, SLOT (mouseTimeout () ) );
	
	bDragging = false;

	QWMatrix wm ( 1, 0, 0, 1, 0, 0 );
	setWorldMatrix ( wm );
	
// Mouse movement and tracking
	viewport ()->setMouseTracking ( true );	
	bObjectDisplayed = false;
	bSelecting = false;
	connect (this, SIGNAL(mouseMove(const QString&)),
			 parent, SLOT(mouseMove(const QString&)));
}
//-------------------------------------------------------------------
SchematuxView::~SchematuxView() {
}
//-------------------------------------------------------------------
void SchematuxView::enableScrollBars ( bool bEnabled ) {
	QScrollView::ScrollBarMode smb = bEnabled ? QScrollView::Auto : QScrollView::AlwaysOff;
	
	setHScrollBarMode ( smb );
	setVScrollBarMode ( smb );
}
//-------------------------------------------------------------------
void SchematuxView::crossHairs ( QCanvasItem * ci ) {
	QPen 			pen ( Qt::red );
	QCanvasLine * 	cl;
	int 			qp_x = ci->x ();
	int 			qp_y = ci->y ();

	cl = new QCanvasLine ( canvas () );
	cl->setPen ( pen );
	cl->setPoints ( qp_x, qp_y - 10,
					qp_x, qp_y + 10 );
	cl->show ();

	cl = new QCanvasLine ( canvas () );
	cl->setPen ( pen );
	cl->setPoints ( qp_x - 10, qp_y,
					qp_x + 10, qp_y );
	cl->show ();
}
//-------------------------------------------------------------------
void SchematuxView::print ( QPainter * p, 
							 int 		height, 
							 int 		width ) {
	drawContents ( 	p, 
					0, 
	   				0, 
					width - 1,
					height - 1 );
}
//-------------------------------------------------------------------
void SchematuxView::keyReleaseEvent ( QKeyEvent * e ) {
	switch ( e->key () ) {
		case Qt::Key_Control:
			bSelecting = false;
			break;
	} /* endif */
}
//-------------------------------------------------------------------
void SchematuxView::keyPressEvent ( QKeyEvent * e ) {
	if ( cilSelected.count () ) {
		QPoint qpMovedBy ( 0, 0 );
		QCanvasItem * ci;
		
		switch ( e->key () ) {
			case Qt::Key_Control:
				bSelecting = true;
				break;
				
			case Qt::Key_Up:
				qpMovedBy.setY ( -1 );
				break;

			case Qt::Key_Down:
				qpMovedBy.setY ( 1 );
				break;

			case Qt::Key_Left:
				qpMovedBy.setX ( -1 );
				break;
				
			case Qt::Key_Right:
				qpMovedBy.setX ( 1 );
				break;

			default:
		// maybe pass it onto the class called for processing
				e->ignore ();
				break;
		} /* endswitch */
		setUpdatesEnabled ( false );
		for ( QCanvasItemList::iterator it = cil.begin(); it != cil.end(); ++it ) {
			ci = *it;
			ci->moveBy ( qpMovedBy.x(), qpMovedBy.y() );
		} /* endfor */
		setUpdatesEnabled ( true );
		repaint ();
		return;
	} /* endif */
	
	switch ( e->key () ) {
	case Qt::Key_Control:
		bSelecting = true;
		break;
		
	case Qt::Key_Up:
		scrollBy ( 0, 1 );
		break;

	case Qt::Key_Down:
		scrollBy ( 0, -1 );
		break;

	case Qt::Key_Left:
		scrollBy ( 1, 0 );
		break;

	case Qt::Key_Right:
		scrollBy ( -1, 0 );
		break;

	default:
		// maybe pass it onto the class called for processing
		e->ignore ();
		return;
	} /* endswitch */
	e->accept ();
	erase ();
	repaintContents ();
}
//-------------------------------------------------------------------
void SchematuxView::contentsMousePressEvent ( QMouseEvent * e ) {
	qpDrag = e->pos ();

	// discover what, if anything, is under the mouse
	cil = canvas ()->collisions ( qpDrag );
	if ( cil.count () ) {
		// move the objects
		bDragging = true;	
		
		if ( cilSelected.count () < 2 ) 
			cilSelected = cil;
		
		else {
			if ( bSelecting )  // ctrl key down
				cilSelected = cilSelected + cil;	
		} /* endif */
		
	} else {
		cilSelected.clear ();
		// move the canvas
		bDragging = true;
		
	} /* endif */
	
// If on empty space, change to a hand icon and move the screen
//	if ( cil.count () == 0 ) {
//		bDragging = true;
//		//QCursor::setShape ( QCursor::SizeAllCursor );
//		QApplication::setOverrideCursor (QCursor (QCursor::SizeAllCursor) );
//		return; 
//	} /* endif */
	
}
//-------------------------------------------------------------------
void SchematuxView::contentsMouseMoveEvent ( QMouseEvent * e ) {
	QPoint qpMovedBy;
	QString qs;
	QCanvasItem * ci;
	
	qpMouse = e->pos ();
	
// Handle the drag operation (if in process)
	if ( bDragging ) {
		qpMovedBy = qpMouse - qpDrag;	
		qpDrag = qpMouse;
		
		if ( cilSelected.count () ) {
			setUpdatesEnabled ( false );
			for ( QCanvasItemList::iterator it = cilSelected.begin(); it != cilSelected.end(); ++it ) {
				ci = *it;
				ci->moveBy ( qpMovedBy.x(), qpMovedBy.y() );
//				updateContents ( ci->boundingRect () );
				
			} /* endfor */
			setUpdatesEnabled ( true );
			canvas()->update();
		} else {
			scrollBy ( qpMovedBy.x() * -1, qpMovedBy.y() * -1);
		} /* endif */
		return;
	} /* endif */
	
// otherwise, report the mouse position
	qs.sprintf ( "%d, %d", qpMouse.x (), qpMouse.y () );
	emit mouseMove ( qs );
		
//	qpDrag = qpNew;
	
	if ( bObjectDisplayed ) {
		emit objList ( "<in motion>" );
		bObjectDisplayed = false;
	} /* endfor */
				
	if ( timMouse.isActive () ) timMouse.stop ();
	//if ( ! bDragging ) 
		timMouse.start ( 500 );
}	
//-------------------------------------------------------------------
void SchematuxView::mouseTimeout (void) {
	QStringList sl;
	QCanvas * c = canvas ();
	
	cil = c->collisions ( qpMouse );
	if ( cil.count () ) {	
		ScheditBitmap * s;
		QCanvasItemList::Iterator it;
		QCanvasItem * so;
		int iRtti;
	
		for ( it = cil.begin (); it != cil.end (); it++ ) {
			so = *it;
			iRtti = so->rtti ();
			switch ( iRtti ) {
			case 1011:				// CanvasPixmap (ScheditBitmap)
				s = (ScheditBitmap *) so;
				sl << s->getID ();
				break;
				
			case 1005:				// ScheditLine
				sl << "Line";
				break;
				
			case 1006:				// ScheditJunction
				sl << "Junction";
				break;
				
			case 1007:				// ScheditAnnotate
				sl << "Annotate";
				break;
				
			case 1009:				// ScheditDocBox
				sl << "DocBox";
				break;
				
			default:
				break;	
			} /* endswitch */
		} /* endfor */
	} else {
		sl << "None";
		
	} /* endif */
	emit objList (sl.join ( "," ) );
	bObjectDisplayed = true;
}
//-------------------------------------------------------------------
void SchematuxView::contentsMouseReleaseEvent ( QMouseEvent * e ) {
	bDragging = false;
	//QCursor::setShape ( QCursor::ArrowCursor );
	QApplication::restoreOverrideCursor ();
	repaintContents ();
}
//-------------------------------------------------------------------
void SchematuxView::slotRedraw ( void ) {
	repaintContents ();
}
//-------------------------------------------------------------------
#include "schematuxview.moc"
