/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _SCHEMATUXVIEW_H_
#define _SCHEMATUXVIEW_H_

#include <qscrollview.h>
#include <kparts/part.h>
#include <schematuxiface.h>
#include <qcanvas.h>
#include <qtimer.h>

class QPainter;
class QCanvas;
class ScheditFile;

/**
 * This is the main view class for Schematux.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * @short Main view
 * @author James Fuller,,, <james@slakware12>
 * @version 0.1
 */
class SchematuxView : public QCanvasView, public SchematuxIface
{
    Q_OBJECT
public:
	/**
	 * Default constructor
	 */
    SchematuxView(QWidget *parent, QCanvas * pCanvas);

	/**
	 * Destructor
	 */
    virtual ~SchematuxView();

    /**
     * Print this view to any medium -- paper or not
     */
    void print(QPainter *, int height, int width);
	
	void enableScrollBars ( bool bFlag );

	
signals:
    /**
 * Use this signal to change the content of the statusbar
	 */
	void signalChangeStatusbar(const QString& text);

    /**
	 * Use this signal to change the content of the statusbar
	 */
	void mouseMove (const QString& text);

    /**
	 * Use this signal to show what is under the mouse
	 */
	void objList (const QString& text);

    /**
     * Use this signal to change the content of the caption
     */
	void setStatusBarText(const QString& text);

public slots:
//    void slotOnURL(const QString& url);
//    void slotSetTitle(const QString& title);
	void slotRedraw (void);
	void mouseTimeout (void);

private:
	QCanvasItemList cil;	
	QCanvasItemList cilSelected;	
	QPoint 			qpDrag;
	bool   			bDragging;
	QTimer 			timMouse;
	QPoint 			qpMouse;
	bool			bObjectDisplayed;
	bool			bSelecting;
	
	void crossHairs ( QCanvasItem * ci );
	
protected:
	void keyPressEvent		 	   ( QKeyEvent   * e );
	void keyReleaseEvent		   ( QKeyEvent   * e );
	void contentsMousePressEvent   ( QMouseEvent * e );
	void contentsMouseMoveEvent    ( QMouseEvent * e );
	void contentsMouseReleaseEvent ( QMouseEvent * e );
};

#endif // _SCHEMATUXVIEW_H_
